package jgui.gui;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.*;
import javax.swing.event.*;

import jgui.actions.*;
import jgui.emailer.util.Contact;
import jgui.model.ContactsModel;

/**
 * 
 * @author Emmanuel DURIN
 *
 */
public class ContactsApplet extends JApplet {

	// bonne pratique de mettre les composants en attributs
	private JList contactsList;
	private JLabel statusBar ;
	private ContactsModel contactsModel;

	private JToolBar toolBar;
	
	private JMenuBar menuBar;
	
	private JMenu fileMenu, editMenu, contactsMenu, helpMenu;
	
	private JMenuItem newFileMenu,openMenu, saveMenu , optionsMenu, exitMenu;
	private JMenuItem newContactMenu, removeContactMenu;
	private JMenuItem cutMenu, copyMenu, pasteMenu;
	private JMenuItem helpContentMenu, aboutMenu; 
	
	private Action newFileAction, openAction, saveAction, optionsAction, 
					exitAction,
				   newContactAction, removeContactAction, cutAction, 
				   pasteAction, copyAction, helpContentAction, aboutAction;
	
	private ContactPanel contactPane;
	
	public ContactsApplet()  {
	//	super("Contacts");
	}
	public void init()
	{
		// 1a : Crer la fentre et ses composants
		Container c = getContentPane();

		contactsModel = new ContactsModel();
		contactsList = new JList(contactsModel);
		contactsList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );		
		
		contactPane = new ContactPanel(contactsModel, null);
		JSplitPane jsp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
		        new JScrollPane(contactsList),
		        contactPane
		        );
		jsp.setResizeWeight(0.5);
		
		c.add(jsp, BorderLayout.CENTER );
		
		statusBar = new JLabel("File :");
		c.add(statusBar, BorderLayout.SOUTH );
		//setSize(400,400);
     //   setDefaultCloseOperation(EXIT_ON_CLOSE);
        
        createActions();
        // 2a
        addMenus();
        addToolBar();
        try {
            // for avoiding the opening of the dialog box at each run
            contactsModel.open(new File("c:\\temp\\jgui-labs\\contacts.data"));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

	private void addToolBar()
	{
	    toolBar = new JToolBar("Menu", JToolBar.HORIZONTAL);
	    toolBar.add( newFileAction);
	    toolBar.add( openAction);
	    toolBar.add( saveAction);
	    
	    toolBar.addSeparator(new Dimension(20,20));

	    toolBar.add( cutAction);
	    toolBar.add( copyAction);
	    toolBar.add( pasteAction);
	    
	    toolBar.addSeparator(new Dimension(20,20));
	    
	    toolBar.add( newContactAction);		
	    toolBar.add( removeContactAction);
	    
	    toolBar.addSeparator(new Dimension(20,20));
	    
	    toolBar.add( helpContentAction);

	    toolBar.setFloatable(true);
	    getContentPane().add( toolBar, BorderLayout.NORTH );
	}
    private void createActions() {

	    newFileAction = new NewFileAction(contactsModel);
	    openAction = new OpenAction(this, statusBar, contactsModel);
	    saveAction = new SaveAction( contactsModel );
	    optionsAction = new OptionsAction(this );
	    // applet
	    //exitAction = new ExitAction( this, contactsModel );
	
	    newContactAction = new AddContactAction(this, contactsModel);
	    removeContactAction = new RemoveContactAction(contactsList, contactsModel);
	
	    cutAction = new CutAction( this );
	    copyAction = new CopyAction(this);
	    pasteAction = new PasteAction( this );
	    
	    helpContentAction = new HelpContentAction ();
	    aboutAction = new AboutAction();
        
	    
		contactsList.addListSelectionListener(new ListSelectionListener(){
			public void valueChanged(ListSelectionEvent event) {
			    //JList l = (JList)event.getSource();
			    // true sur le premier vnement de la srie
			    if ( ! event.getValueIsAdjusting()){
			        selectionChanged();
			    }
			}
		});
    }	
    
    public void addContact() {
        contactsList.setSelectedIndex( contactsModel.getSize()-1 );
    }    

    protected void selectionChanged()
    {
        Contact c = (Contact)contactsList.getSelectedValue();
        if ( c != null ){
            contactPane.setContact(c);
        }        
    }

	private void addMenus() {
        
        menuBar = new JMenuBar();
                
        saveMenu = new JMenuItem(saveAction);
        openMenu = new JMenuItem(openAction);
        newFileMenu = new JMenuItem(newFileAction);
        optionsMenu = new JMenuItem(optionsAction);
        exitMenu = new JMenuItem(exitAction);
        
        newContactMenu  = new JMenuItem(newContactAction);        
        removeContactMenu  = new JMenuItem(removeContactAction);
        
        cutMenu = new JMenuItem(cutAction);
        
        copyMenu = new JMenuItem(copyAction);
        
        pasteMenu  = new JMenuItem(pasteAction);
 
        helpContentMenu = new JMenuItem(helpContentAction);

        aboutMenu  = new JMenuItem(aboutAction);
        
        fileMenu = new JMenu("File");
        fileMenu.setMnemonic(KeyEvent.VK_F);
        
        editMenu = new JMenu("Edit");
        editMenu.setMnemonic(KeyEvent.VK_E);        
        
        contactsMenu = new JMenu("Contacts");
        contactsMenu.setMnemonic(KeyEvent.VK_C);
        
        helpMenu = new JMenu("Help");
        helpMenu.setMnemonic(KeyEvent.VK_H);

        fileMenu.add( newFileMenu );
        fileMenu.add( openMenu );
        fileMenu.add( saveMenu );
        fileMenu.add( optionsMenu );
        fileMenu.add( exitMenu );

        editMenu.add( cutMenu );
        editMenu.add( copyMenu );
        editMenu.add( pasteMenu );
        
        contactsMenu.add( newContactMenu );
        contactsMenu.add( removeContactMenu );

        helpMenu.add( helpContentMenu );
        helpMenu.add( aboutMenu );
        
        menuBar.add(fileMenu);
        menuBar.add(editMenu);
        menuBar.add(contactsMenu);        
        menuBar.add(helpMenu);
        
        setJMenuBar( menuBar );
    }
//		Applet
//    public static void main(String[] args) {
//        ContactLocale.load();
//		ContactsApplet cf = new ContactsApplet();
//		cf.setVisible(true);
//	}
}
