import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.util.*;

import javax.swing.*;
/*
 * Created on 8 fvr. 2006
 */

/**
 * @author Emmanuel DURIN
 *
 */
public class DrawingLabel extends JLabel implements MouseListener {
    
    private int xCenter, yCenter;
    private Color c;
    
    public DrawingLabel(Color backGroundColor, 
            Color foreGroundColor,
            int x, int y, int width, int height, int level) {
        setBackground( backGroundColor );
        setVerticalAlignment(JLabel.TOP);
        setHorizontalAlignment(JLabel.CENTER);
        setText(getlabelFromInt(level));
        this.c = foreGroundColor;
        setOpaque(false);
        setBounds(x,y,width,height);
        xCenter = width / 2;
        yCenter = height / 2;
        Thread t = new Thread( new Runnable(){
            public void run() {
                while(true){
                    try {
                        Thread.sleep(1000);
                        // Cette mthode fait partie des composants graphiques de Java
                        // Elle demande au composant de se redessiner
                        // Un message de dessin est empil pour le composant
                        repaint();
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }
        });
        t.start();
        
        addMouseListener(this);
    }
    
    private String getlabelFromInt(int level) {
        // Pas efficace, mieux plus bas
        //JLayeredPane jp = new JLayeredPane();
        //Class c = jp.getClass();
        
        // obtenir la classe
        Class c = JLayeredPane.class;
        
        Field [] fields = c.getDeclaredFields();
        // on fait une boucle sur tous les champs de la classe
        try {
            
            for (int i = 0; i < fields.length; i++) {
                Field field = fields[i];
//                System.out.print(field.getName() + " - " + level );
                // Vrifiication du caractre public sinon IllegalAccessException					                		
//                if ( (field.getModifiers() & Modifier.PUBLIC ) == Modifier.PUBLIC )
//                    System.out.print(" - " + field.get(null) );
//                System.out.println();
                int modifiers = field.getModifiers();
                // on vrifie pour chaque champ que 
                // 1) C'est bien un champ de classe Integer
                // 2) que le modicateur final est appliqu
                // 3) que le modicateur static est appliqu
                // 4) que le modicateur public est appliqu
                // 5) comparaison de valeur ncessaire !
                if ( field.getType().equals(java.lang.Integer.class )
                        && (modifiers & Modifier.FINAL) == Modifier.FINAL 
                        && (modifiers & Modifier.STATIC) == Modifier.STATIC 
                        && (modifiers & Modifier.PUBLIC) == Modifier.PUBLIC 
                        // get permet d'obtenir la valeur de l'attribut pass
                        // Le paramtre pass  get est l'objet sur le lequel 
                        // on fait la requte
                        // Comme la mthode est statique, on ne passe pas d'objet.
                        // On passe null
                        && level == ((Integer)field.get(null)).intValue()
                ){
                    // trouv le bon nom
//                    System.out.println("trouv !");
                    return field.getName();
                }
            }
            
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }        
        return "Pas trouv";
    }
    
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        setForeground(c);
        
        // Calendar est mieux que java.util.Date, car localis
        Calendar time = Calendar.getInstance(Locale.FRANCE); 
        long seconds = time.get(Calendar.SECOND);
        long minutes = time.get(Calendar.MINUTE); 
        long hours = time.get(Calendar.HOUR);
        
        // On utilise l'objet Graphics (g) pass en paramtre de la fonction
        // paintComponent pour faire des dessins
        
        //Secondes
        g.drawLine(xCenter, yCenter, 
                xCenter + (int)(xCenter*Math.cos( -seconds*2*Math.PI/60 +Math.PI/2)) , 
                yCenter - (int)(yCenter*Math.sin( -seconds*2*Math.PI/60 +Math.PI/2)));
        // Minutes
        g.drawLine(xCenter, yCenter, 
                xCenter + (int)(xCenter*Math.cos( -minutes*2*Math.PI/60 +Math.PI/2)*3/4) , 
                yCenter - (int)(yCenter*Math.sin( -minutes*2*Math.PI/60 +Math.PI/2))*3/4);
        // Heures
        g.drawLine(xCenter, yCenter, 
                xCenter + (int)(xCenter*Math.cos( -hours*2*Math.PI/12 +Math.PI/2)/2) , 
                yCenter - (int)(yCenter*Math.sin( -hours*2*Math.PI/12 +Math.PI/2))/2);        
        //g.fillRect(0,0,getWidth(),getHeight());
    }

    public void mouseClicked(MouseEvent e) {
        System.out.println("click");
    }

    public void mousePressed(MouseEvent e) {
        System.out.println("pressed");
    }

    public void mouseReleased(MouseEvent e) {
        
    }

    public void mouseEntered(MouseEvent e) {
        
    }

    public void mouseExited(MouseEvent e) {
        
    }
}
