import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.*;

import javax.swing.*;
import jgui.emailer.util.*;

/**
 * 
 * @author Emmanuel DURIN
 *
 */
public class ContactsFrame extends JFrame {

	// bonne pratique de mettre les composants en attributs
	private JList contactsList;
	private JLabel statusBar ;
	private DefaultListModel contactsModel;

	private JMenuBar menuBar;
	private JMenuItem openMenu, saveMenu, exitMenu, newContactMenu, openContactMenu, removeContactMenu;
	private JMenu fileMenu, contactsMenu;	
	
	public ContactsFrame()  {
		super("Contacts");
		// 1a : Crer la fentre et ses composants
		Container c = getContentPane();

		contactsModel = new DefaultListModel();
		contactsList = new JList(contactsModel);
		contactsList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );		
		c.add(new JScrollPane(contactsList), BorderLayout.CENTER );
		
		statusBar = new JLabel("File :");
		c.add(statusBar, BorderLayout.SOUTH );
		setSize(400,300);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        
        // 2a
        addMenus();
        
        // for avoiding the opening of the dialog box at each run
		readFile(new File("c:\\temp\\jgui-labs\\contacts.data"));        
	}

    /**
     * 
     */
    private void addActions() {
		// 1b : Ajouter les vnements (listeners)
      openMenu.addActionListener(
      	new ActionListener(){
      		public void actionPerformed(ActionEvent event)
      		{
      			readFile();
      		}
      });

      exitMenu.addActionListener(
          	new ActionListener(){
          		public void actionPerformed(ActionEvent event)
          		{
          			dispose();
          		}
          });
      openContactMenu.addActionListener( new java.awt.event.ActionListener(){
        public void actionPerformed(  java.awt.event.ActionEvent event)
        {
            Contact c = (Contact)contactsList.getSelectedValue();
            if ( c!= null ){
                ContactDialog dialog = new ContactDialog(ContactsFrame.this, c);
                dialog.setVisible(true);
            }
        }
      });
      
      this.contactsList.addMouseListener(
              new MouseAdapter(){
                  public void mouseClicked( MouseEvent event)
                  {
                      if ( event.getClickCount() == 2){
                          int index = contactsList.locationToIndex(event.getPoint());
                          Contact c = (Contact)contactsModel.get(index);
                          if ( c != null ){
                              // == la classe Anonyme de MouseListener.
                              // this n'est donc pas une JFrame.
                              // Il faut remonter  l'instance de classe imbriquante
                              // La syntaxe est :
                              // ClasseImbriquante.this 
                              // plutt que :
                              // this                          	
                              ContactDialog dialog = 
                                  new ContactDialog(ContactsFrame.this, c);
                              dialog.setVisible(true);
                              
                          }
                      }
                  }
      });
    }	
	
	/**
     * 
     */
    private void addMenus() {
        menuBar = new JMenuBar();
        
        openMenu = new JMenuItem("Open...");
        openMenu.setMnemonic(KeyEvent.VK_O);
        saveMenu = new JMenuItem("Save");
        saveMenu.setMnemonic(KeyEvent.VK_S);
        exitMenu = new JMenuItem("Exit");
        exitMenu.setMnemonic(KeyEvent.VK_X);
        
        newContactMenu = new JMenuItem("New Contact");
        newContactMenu.setMnemonic(KeyEvent.VK_N);
       
        openContactMenu = new JMenuItem("Open Contact");
        openContactMenu.setMnemonic(KeyEvent.VK_O);
        
        removeContactMenu  = new JMenuItem("Remove Contact");
        removeContactMenu.setMnemonic(KeyEvent.VK_R);
        
        fileMenu = new JMenu("File");
        fileMenu.setMnemonic(KeyEvent.VK_F);
        contactsMenu = new JMenu("Contacts");
        contactsMenu.setMnemonic(KeyEvent.VK_C);
        
        fileMenu.add( openMenu );
        fileMenu.add( saveMenu );
        fileMenu.add( exitMenu );

        contactsMenu.add( newContactMenu );
        contactsMenu.add( openContactMenu );        
        contactsMenu.add( removeContactMenu );
        
        menuBar.add(fileMenu);
        menuBar.add(contactsMenu);
        
        setJMenuBar( menuBar );
        
        addActions();
    }

    private void readFile()
	{
		JFileChooser jfc = new JFileChooser();
		int result = jfc.showOpenDialog(this);
		if (result != JFileChooser.APPROVE_OPTION){
		    return;
		}
	    File selectedFile = jfc.getSelectedFile();

	    readFile( selectedFile );
	}

    private void readFile(File selectedFile) {
		try{
			statusBar.setText("... ");
			Vector v = Toolbox.loadVector( selectedFile );
			statusBar.setText( selectedFile.getCanonicalPath());
			
			// hasNext() retourne vrai tant qu'on peut avancer
			for (Iterator iter = v.iterator(); iter.hasNext();) {
			    // next() fait avancer  et retourne un objet
				contactsModel.addElement( iter.next() );
			}
			
			// autre manire de remplir le modle, sans Iterator
//			int i = 0;
//			while ( i < v.size() ){
//			    contactsModel.addElement( v.get(i) );
//				i++;
//			}
			
		}catch( Exception ex ){
			ex.printStackTrace();
		}	   
    }

    public static void main(String[] args) {
		ContactsFrame cf = new ContactsFrame();
		cf.setVisible(true);
	}
}
