package jgui.gui;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;

import jgui.emailer.util.Contact;
import jgui.model.ContactsModel;

/**
 * 
 * @author Emmanuel DURIN
 *
 */
public class ContactsFrame extends JFrame {

	// bonne pratique de mettre les composants en attributs
	private JList contactsList;
	private JLabel statusBar ;
	private ContactsModel contactsModel;

	private JMenuBar menuBar;
	private JMenuItem openMenu, saveMenu, exitMenu, newContactMenu, openContactMenu, removeContactMenu;
	private JMenu fileMenu, contactsMenu;	
	
	public ContactsFrame()  {
		super("Contacts");
		// 1a : Crer la fentre et ses composants
		Container c = getContentPane();

		contactsModel = new ContactsModel();
		contactsList = new JList(contactsModel);
		contactsList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );		
		c.add(new JScrollPane(contactsList), BorderLayout.CENTER );
		
		statusBar = new JLabel("File :");
		c.add(statusBar, BorderLayout.SOUTH );
		setSize(400,300);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        
        // 2a
        addMenus();
        
        // for avoiding the opening of the dialog box at each run
		readFile(new File("c:\\temp\\jgui-labs\\contacts.data"));        
	}

    /**
     * 
     */
    private void addActions() {
		// 1b : Ajouter les vnements (listeners)
      openMenu.addActionListener(
      	new ActionListener(){
      		public void actionPerformed(ActionEvent event)
      		{
      			readFile();
      		}
      });
      saveMenu.addActionListener(
		new ActionListener(){
			public void actionPerformed(ActionEvent event)
			{
				saveFile();
			}
		});

      exitMenu.addActionListener(
          	new ActionListener(){
          		public void actionPerformed(ActionEvent event)
          		{
          			dispose();
          		}
          });
      
      openContactMenu.addActionListener( new java.awt.event.ActionListener(){
        public void actionPerformed(  java.awt.event.ActionEvent event)
        {
            Contact c = (Contact)contactsList.getSelectedValue();
            if ( c!= null ){
                ContactDialog dialog = new ContactDialog(ContactsFrame.this, c);
                dialog.setVisible(true);
            }
        }
      });

      newContactMenu.addActionListener( new java.awt.event.ActionListener(){
          public void actionPerformed(  java.awt.event.ActionEvent event)
          {
              contactsModel.add(new Contact() );
          }
        });

      removeContactMenu.addActionListener( new java.awt.event.ActionListener(){
          public void actionPerformed(  java.awt.event.ActionEvent event)
          {
              int selectedIndex = contactsList.getSelectedIndex(); 
              if ( selectedIndex != -1 )
                  contactsModel.remove(selectedIndex);
          }
        });

      
      this.contactsList.addMouseListener(
              new MouseAdapter(){
                  public void mouseClicked( MouseEvent event)
                  {
                      if ( event.getClickCount() == 2){
                          int index = contactsList.locationToIndex(event.getPoint());
                          Contact c = (Contact)contactsModel.getElementAt(index);
                          if ( c != null ){
                              ContactDialog dialog = new ContactDialog(ContactsFrame.this, c);
                              dialog.setVisible(true);
                          }
                      }
                  }
      });
    }	
	
	/**
     * 
     */
    private void addMenus() {
        menuBar = new JMenuBar();
        
        openMenu = new JMenuItem("Open...");
        openMenu.setMnemonic(KeyEvent.VK_O);
        saveMenu = new JMenuItem("Save");
        saveMenu.setMnemonic(KeyEvent.VK_S);
        exitMenu = new JMenuItem("Exit");
        exitMenu.setMnemonic(KeyEvent.VK_X);
        
        newContactMenu = new JMenuItem("New Contact");
        newContactMenu.setMnemonic(KeyEvent.VK_N);
       
        openContactMenu = new JMenuItem("Open Contact");
        openContactMenu.setMnemonic(KeyEvent.VK_O);
        
        removeContactMenu  = new JMenuItem("Remove Contact");
        removeContactMenu.setMnemonic(KeyEvent.VK_R);
        
        fileMenu = new JMenu("File");
        fileMenu.setMnemonic(KeyEvent.VK_F);
        contactsMenu = new JMenu("Contacts");
        contactsMenu.setMnemonic(KeyEvent.VK_C);
        
        fileMenu.add( openMenu );
        fileMenu.add( saveMenu );
        fileMenu.add( exitMenu );

        contactsMenu.add( newContactMenu );
        contactsMenu.add( openContactMenu );        
        contactsMenu.add( removeContactMenu );
        
        menuBar.add(fileMenu);
        menuBar.add(contactsMenu);
        
        setJMenuBar( menuBar );
        
        addActions();
    }

    private void saveFile()
	{
	    try {
            contactsModel.save();
        } catch (IOException e) {
            // en  attente de miex signaler les erreurs
            e.printStackTrace();
        }
	}
		
    private void readFile()
	{
		JFileChooser jfc = new JFileChooser();
		int result = jfc.showOpenDialog(this);
		if (result != JFileChooser.APPROVE_OPTION){
		    return;
		}
	    File selectedFile = jfc.getSelectedFile();
	    readFile( selectedFile );
	}

    private void readFile(File selectedFile) {
		statusBar.setText("... ");
		try {
		    contactsModel.open( selectedFile  );		    
            statusBar.setText( selectedFile.getCanonicalPath());
        } catch (Exception e) {
            // en  attente de miex signaler les erreurs            
            e.printStackTrace();
        }
    }

    public static void main(String[] args) {
		ContactsFrame cf = new ContactsFrame();
		cf.setVisible(true);
	}
}
