package jgui.gui;
import java.awt.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

import jgui.emailer.util.Contact;
import jgui.model.*;
/*
 * Created on 17 juin 2005
 */

/**
 * @author Emmanuel DURIN
 *
 */
public class ContactPanel extends JPanel implements DocumentListener {
    
    private JLabel labelFirstname = new JLabel("Firstname : ");
    private JLabel labelLastname = new JLabel("Lastname : ");
    private JLabel labelEmail = new JLabel("Email : ");
    
    private JTextField textFirstname = new JTextField(15);
    private JTextField textLastname = new JTextField(15);;
    private JTextField textEmail = new JTextField(15);
    
    private Contact contact;
    
    private ArrayList<ContactListener> contactListeners 
            = new ArrayList<ContactListener>();
    private boolean isModificationFromUser = true;
    
    public ContactPanel(Contact contact) {
        setContact( contact ) ;
        
        // le Document est le modle du textField
        textFirstname.getDocument().addDocumentListener( this );
        textLastname.getDocument().addDocumentListener( this );
        textEmail.getDocument().addDocumentListener( this );
        
        setGridBagLayout( );
    }
    
    private void applyChanges() {
        // il faut passer dans un mode o on n'est plus  l'coute des modifications
        // provenant des modifs faites dans les zones de saisie
        if ( isModificationFromUser ){        
            if ( contact != null){
                contact.setFirstname(textFirstname.getText( ));
                contact.setLastname(textLastname.getText( ));
                contact.setEmailAddress(textEmail.getText( ));
                
                for (Iterator<ContactListener> i = contactListeners.iterator();
                i.hasNext(); ) {
                    ContactListener listener = i.next();
                    listener.contactChanged(
                            new ContactEvent(contact,this)
                    );
                }          
            }
        }
    }
    
    public void setContact( Contact c )
    {
        this.contact = c;
        reset();
    }
    
    private void reset() {
        isModificationFromUser = false;
        if ( contact != null){
            textFirstname.setText( contact.getFirstname());
            textLastname.setText( contact.getLastname());
            textEmail.setText( contact.getEmailAddress());
        }else{
            textFirstname.setText( "");
            textLastname.setText( "" );
            textEmail.setText( "" );
        }
        isModificationFromUser = true;
    }
    
    
    public void setGridBagLayout( )
    {
        setLayout( new GridBagLayout() );
        
        GridBagConstraints gbc = new GridBagConstraints();
        
        gbc.insets.top = gbc.insets.bottom = 5;
        gbc.insets.left = gbc.insets.right = 5;
        gbc.gridx = gbc.gridy = 0;  
        add(labelFirstname, gbc);
        
        gbc.gridy ++;
        add(labelLastname, gbc);
        
        gbc.gridy ++;
        add(labelEmail, gbc);
        
        gbc.gridx = 1;
        gbc.gridy = 0;  
        add(textFirstname, gbc);
        
        gbc.gridy ++;
        add(textLastname, gbc);
        
        gbc.gridy ++;
        add(textEmail, gbc);
    }    
    
    public void addContactListener( ContactListener listener )
    {
        contactListeners.add( listener );
    }
    
    public void removeContactListener( ContactListener listener )
    {
        contactListeners.remove( listener );
    }    
    
    // Suivent les 3 mthodes du DocumentListener
    // qui correspondent  la modif d'un caractre dans les JTextField
    public void insertUpdate(DocumentEvent e) {
        applyChanges();
    }
    public void removeUpdate(DocumentEvent e) {
        applyChanges();    }
    public void changedUpdate(DocumentEvent e) {
        applyChanges();
    }
    
    public static void main(String[] args) {
        JFrame frame = new JFrame();
        frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        
        frame.setContentPane( new ContactPanel( null) );
        frame.pack();
        frame.setVisible(true);
    }    
}