package jgui.gui;
import java.awt.*;
import java.io.File;

import javax.swing.*;
import javax.swing.event.*;

import jgui.emailer.util.*;
import jgui.model.ContactsModel;
/*
 * Created on 17 juin 2005
 */

/**
 * @author Emmanuel DURIN
 *
 */
public class ContactPanel extends JPanel {

    private JLabel labelFirstname = new JLabel("Firstname : ");
    private JLabel labelLastname = new JLabel("Lastname : ");
    private JLabel labelEmail = new JLabel("Email : ");
    private JTextField textFirstname = new JTextField(15);
    private JTextField textLastname = new JTextField(15);;
    private JTextField textEmail = new JTextField(15);
        
    private JButton buttonApply = new JButton("Apply");
    private JButton buttonReset = new JButton("Reset");
    private PictureBean picture;
    
    private Contact contact;
    
    private ContactsModel model;
    
    private boolean isDirty;
    
    public ContactPanel(ContactsModel model, Contact contact) {
        setContact( contact ) ;
        reset();
        this.model = model;

        setGridBagLayout( );

        addListeners();
    }
    /**
     * 
     * cette mthode est appele par la Frame lorsque la slection change
     */
    public void setContact( Contact c )
    {
        if ( isDirty ){
            if (JOptionPane.showConfirmDialog(this, 
                    "Do you want to apply changes to the modified contact ("+ this.contact.getFirstname() + ' ' + this.contact.getLastname()+")?",
                    "Caution",                    
                    JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION){
                applyChanges();
            }
        }
        this.contact = c;

        reset();
       
        buttonApply.setEnabled( c != null );        
        buttonReset.setEnabled( c != null );
    }
    
    private void setGridBagLayout() {
        setLayout( new GridBagLayout() );
        GridBagConstraints gbc = new GridBagConstraints();
        	
        gbc.anchor = GridBagConstraints.EAST;
        gbc.insets.bottom = gbc.insets.top = 5; 
        gbc.insets.left = gbc.insets.right = 10;
        gbc.gridx = gbc.gridy = 0;
        add(labelFirstname, gbc);
        gbc.gridy ++;
        add(labelLastname, gbc);
        gbc.gridy ++;
        add(labelEmail,gbc);

        gbc.gridy ++;
        gbc.gridwidth = 2;
        gbc.anchor = GridBagConstraints.CENTER;
        picture = new PictureBean();
        picture.setDirectory(".."+ File.separator + "pictures");
        picture.setImageIcon( new ImageIcon(".."+File.separator + "pictures" + File.separator +"Corinne.jpg"));
        add(picture, gbc);
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.gridy ++;
        add(buttonApply,gbc);

        gbc.anchor = GridBagConstraints.WEST;
        gbc.gridy = 0;
        gbc.gridx = 1;
        add(textFirstname,gbc);
        gbc.gridy ++;        
        add(textLastname,gbc);
        gbc.gridy ++;
        add(textEmail,gbc);
        gbc.gridy += 2;                
        add(buttonReset,gbc);
        
        textFirstname.setMinimumSize( textFirstname.getPreferredSize() );
        textLastname.setMinimumSize( textLastname.getPreferredSize() );
        textEmail.setMinimumSize( textEmail.getPreferredSize() );
    }

    private void addListeners()
    {
        DocumentListener docListener = new DocumentListener(){
            public void changedUpdate(DocumentEvent event) {
                if ( contact!= null )
                    isDirty = true;                
            }
            public void insertUpdate(DocumentEvent event) {
                if ( contact!= null )
                    isDirty = true;
            }
            public void removeUpdate(DocumentEvent event) {
                if ( contact!= null )
                    isDirty = true;
            }
        };
        textFirstname.getDocument().addDocumentListener(docListener);
        textLastname.getDocument().addDocumentListener(docListener);
        textEmail.getDocument().addDocumentListener(docListener);
        buttonApply.addActionListener( new java.awt.event.ActionListener(){
            public void actionPerformed(  java.awt.event.ActionEvent event)
            {
                applyChanges();
            }
        });
        
        buttonReset.addActionListener( new java.awt.event.ActionListener(){
            public void actionPerformed(  java.awt.event.ActionEvent event)
            {
                reset();
            }
        });        
    }
    private void applyChanges() {
        if ( contact != null){
	        contact.setFirstname(textFirstname.getText( ));
	        contact.setLastname(textLastname.getText( ));
	        contact.setEmailAddress(textEmail.getText( ));
	        model.contactChanged(contact);
        }
        isDirty = false;        
    }
    private void reset() {
        if ( contact != null){
	        textFirstname.setText( contact.getFirstname());
	        textLastname.setText( contact.getLastname());
	        textEmail.setText( contact.getEmailAddress());
        }else{
	        textFirstname.setText( "");
	        textLastname.setText( "" );
	        textEmail.setText( "" );
        }
        isDirty = false;        
    }
}