/*
 * Created on 24 juin 2005
 */
package jgui.model;

import java.io.*;
import java.io.File;
import java.util.*;
import java.util.Vector;

import javax.swing.AbstractListModel;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import jgui.emailer.util.*;
import jgui.emailer.util.Contact;

/**
 * @author Emmanuel DURIN
 *
 */
public class ContactsModel extends AbstractListModel implements TableModel{

    private Vector contacts = new Vector();
    private File f;
    private boolean isDirty;

    /* (non-Javadoc)
     * @see javax.swing.ListModel#getSize()
     */
    public int getSize() {
        return contacts.size();
    }

    /* (non-Javadoc)
     * @see javax.swing.ListModel#getElementAt(int)
     */
    public Object getElementAt(int index) {
        return contacts.get(index);
    }

    public void add( Contact c )
    {
        isDirty = true;        
        contacts.add(c);
        fireIntervalAdded(this,contacts.size()-1,contacts.size()-1);
    }
    public Object remove(int index) {
        isDirty = true;    
        if ( index >= 0 && index < contacts.size() ){
            isDirty = true;
            fireIntervalRemoved(this,index, index);            
        }
        return contacts.remove(index);
    }
    public boolean remove(Contact c ) {
        int index = contacts.indexOf(c);
        if ( index != -1 ){
            isDirty = true;
            fireIntervalRemoved(this, index, index);
        }
        return contacts.remove(c);
    }
    
    public void contactChanged( Contact c )
    {
        int index = contacts.indexOf(c);
        if ( index != -1 ){
            isDirty = true;
            fireContentsChanged(this, index, index);
        }
    }

    public void save() throws IOException
    {
        Toolbox.saveVector(contacts, f);
        isDirty = false;
    }
    
    public void newFile()
    {
        f = null;
        isDirty = false;
        if (contacts != null){
            if ( contacts.size() > 0) 
                fireIntervalRemoved(this,0,contacts.size()-1 );            
	        contacts.clear();
        }
    }
    
    // Les exceptions doivent remonter jusqu' l'IHM
    // car on doit tre capable de les signaler  l'utilisateur
    public void open(File f) throws OptionalDataException, FileNotFoundException, ClassNotFoundException, IOException
    {
        newFile();
        isDirty = false;        
        this.f = f;
		Vector v = Toolbox.loadVector( f );

		for (Iterator iter = v.iterator(); iter.hasNext();) {
			add( (Contact)iter.next() );
		}
    }

    public File getFile() {
        return f;
    }
    public void setFile(File selectedFile) {
        this.f = selectedFile;
    }    
    
    /**
     * @return Returns the isDirty.
     */
    public boolean isDirty() {
        return isDirty;
    }
    /**
     * @param isDirty The isDirty to set.
     */
    public void setDirty(boolean isDirty) {
        this.isDirty = isDirty;
    }

    public int getColumnCount() {
        return 3;
    }
    public int getRowCount() {
        return contacts.size();
    }

    public boolean isCellEditable(int arg0, int arg1) {
        return true;
    }

    public Class getColumnClass(int col) {
//        String s = "";
//        return s.getClass();
        return String.class;
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        Contact c = (Contact)contacts.get(rowIndex);
        switch ( columnIndex ){
        	case 0:
        	    return c.getFirstname();
        	case 1:
        	    return c.getLastname();
        	case 2:
        	    return c.getEmailAddress();        	    
        }
//        assert false;
        return null;
    }

    public void setValueAt(Object value, int rowIndex, int columnIndex) {
        Contact c = (Contact)contacts.get(rowIndex);
        switch ( columnIndex ){
        	case 0:
        	    c.setFirstname( value.toString() );
        	    break;
        	case 1:
        	    c.setLastname(value.toString());
        	    break;
        	case 2:
        	    c.setEmailAddress(value.toString());
        	    break;
        }
        int  index = contacts.indexOf(value);
	    fireContentsChanged(value, index, index );        
    }

    /* (non-Javadoc)
     * @see javax.swing.table.TableModel#getColumnName(int)
     */
    public String getColumnName(int col) {
        String [] names = {"FirstName", "Lastname", "Email" };
        return names[col];
    }

    Vector tableModelListeners = new Vector();
    public void addTableModelListener(TableModelListener l) {
        tableModelListeners.add(l);
        
    }

    public void removeTableModelListener(TableModelListener l) {
        tableModelListeners.remove(l);
        
    }
}