package jgui.gui;
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;
import javax.swing.event.*;

import jgui.actions.*;
import jgui.emailer.util.Contact;
import jgui.gui.beans.Portrait;
import jgui.gui.datatransfer.dnd.ContactSelection;
import jgui.gui.locale.ContactLocale;
import jgui.model.*;

/**
 * 
 * @author Emmanuel DURIN
 *
 */
public class ContactsFrame extends JFrame {

    // bonne pratique de mettre les composants en attributs
//    private JButton openButton, saveButton, exitButton;
    private JList contactsList;
    private JLabel statusBar ;
    private ContactsModel contactsModel;

    private JToolBar toolBar;
    
    private JMenuBar menuBar;
    
    private JMenu fileMenu, editMenu, contactsMenu, helpMenu;
    
    private JMenuItem newFileMenu,openMenu, saveMenu,  optionsMenu, exitMenu;
    private JMenuItem newContactMenu, removeContactMenu;
    private JMenuItem cutMenu, copyMenu, pasteMenu;
    private JMenuItem helpContentMenu, aboutMenu; 
    
    private Action newFileAction, openAction, saveAction,  optionsAction, 
                    exitAction,
                   newContactAction, removeContactAction, cutAction, 
                   pasteAction, copyAction, helpContentAction, aboutAction;
    
    private ContactPanel contactPane;
    private JTable table;
    public ContactsFrame()  {
        super("Contacts");
        // 1a : Crer la fentre et ses composants
        Container c = getContentPane();

        contactsModel = new ContactsModel();
        contactsList = new JList(contactsModel);
        contactsList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );       
        
        contactPane = new ContactPanel(contactsModel, null);
        JSplitPane contactAndListPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                new JScrollPane(contactsList),
                contactPane
                );
        contactAndListPane.setResizeWeight(0.5);

        table = new JTable(contactsModel);
        table.setDefaultRenderer(Portrait.class, new PortraitRender());
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION );
        JSplitPane horizontallySplittedPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                contactAndListPane,
                new JScrollPane(table));        
        
        c.add(horizontallySplittedPane, BorderLayout.CENTER );
        
        statusBar = new JLabel("File :");
        c.add(statusBar, BorderLayout.SOUTH );
        //setSize(400,400);
        contactsList.setSize(200,200);
        table.setSize(200,200);
        pack();
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        
        createActions();
        // 2a
        addMenus();
        addToolBar();

        try {
            // for avoiding the opening of the dialog box at each run
            contactsModel.open(new File(".." + File.separator +"contacts.data"));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void addToolBar()
    {
        toolBar = new JToolBar("Menu", JToolBar.HORIZONTAL);
        toolBar.add( newFileAction);
        toolBar.add( openAction);
        toolBar.add( saveAction);
        
        toolBar.addSeparator(new Dimension(20,20));

        toolBar.add( cutAction);
        toolBar.add( copyAction);
        toolBar.add( pasteAction);
        
        toolBar.addSeparator(new Dimension(20,20));
        
        toolBar.add( newContactAction);     
        toolBar.add( removeContactAction);
        
        toolBar.addSeparator(new Dimension(20,20));
        
        toolBar.add( helpContentAction);

        toolBar.setFloatable(true);
        getContentPane().add( toolBar, BorderLayout.NORTH );
    }
    private void createActions() {

        newFileAction = new NewFileAction(contactsModel);
        openAction = new OpenAction(this, statusBar, contactsModel);
        saveAction = new SaveAction(contactsModel );

        optionsAction = new OptionsAction(this);
        exitAction = new ExitAction( this, contactsModel );
    
        newContactAction = new AddContactAction(this, contactsModel);
        removeContactAction = new RemoveContactAction( contactsList, contactsModel);
    
        cutAction = new CutAction(this );
        copyAction = new CopyAction(this);
        pasteAction = new PasteAction( this );
        
        helpContentAction = new HelpContentAction ();
        aboutAction = new AboutAction();
        
        contactsList.addListSelectionListener(new ListSelectionListener(){
            public void valueChanged(ListSelectionEvent event) {
                //JList l = (JList)event.getSource();
                // true sur le premier vnement de la srie
                if ( ! event.getValueIsAdjusting()){
                    selectionChanged();
                }
            }
        });
        
        KeyStroke keystroke = KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK);
        InputMap imap = contactsList.getInputMap(JComponent.WHEN_FOCUSED);
        ActionMap amap = contactsList.getActionMap();
        imap.put(keystroke, "listcopy");
        amap.put("listcopy", new AbstractAction(){
            public void actionPerformed(ActionEvent e) {
                System.out.println("copy copy");
                Contact c = (Contact)contactsList.getSelectedValue();
                if ( c != null){
                    ContactSelection selection = new ContactSelection( c );
                    
                    Toolkit toolkit = Toolkit.getDefaultToolkit();
                    Clipboard clipboard = toolkit.getSystemClipboard();
                    // le 2me param est du type ClipboardOwner.
                    // Il devrait implmenter lostOwnership(Clipboard clipboard, Transferable contents) 
                    clipboard.setContents(selection, null );
                }
            }
        });
        keystroke = KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK);
        imap.put(keystroke, "listpaste");
        amap.put("listpaste", new AbstractAction(){
            public void actionPerformed(ActionEvent e) {
                System.out.println("paste paste");
                Toolkit toolkit = Toolkit.getDefaultToolkit();
                Clipboard clipboard = toolkit.getSystemClipboard();
                
                //Contact c = null;
                try {
                    DataFlavor [] flavors = clipboard.getAvailableDataFlavors();
                    for (int i = 0; i < flavors.length; i++) {
                        System.out.println( flavors[i].toString() );
                    }
                    Object o = clipboard.getData( ContactSelection.javaReferenceFlavor );
                    Contact c = (Contact)o;
                    if ( c == null ){
                        c = (Contact)clipboard.getData( ContactSelection.serializedFlavor );
                  
                    }
                    if ( c != null){
                        // le 2me param est du type ClipboardOwner.
                        // Il devrait implmenter lostOwnership(Clipboard clipboard, Transferable contents) 
                        contactsModel.add(c);
                    }
                } catch (UnsupportedFlavorException e1) {
                    e1.printStackTrace();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }                    
            }
        });        
    }   
    
    public void addContact() {
        contactsList.setSelectedIndex( contactsModel.getSize()-1 );
    }    

    protected void selectionChanged()
    {
        Contact c = (Contact)contactsList.getSelectedValue();
        if ( c != null ){
            contactPane.setContact(c);
        }        
    }

    private void addMenus() {
        menuBar = new JMenuBar();
                
        saveMenu = new JMenuItem(saveAction);
        
        openMenu = new JMenuItem(openAction);
        newFileMenu = new JMenuItem(newFileAction);
        optionsMenu = new JMenuItem(optionsAction);
        exitMenu = new JMenuItem(exitAction);
        
        newContactMenu  = new JMenuItem(newContactAction);        
        removeContactMenu  = new JMenuItem(removeContactAction);
        
        cutMenu = new JMenuItem(cutAction);
        copyMenu = new JMenuItem(copyAction);
        pasteMenu  = new JMenuItem(pasteAction);
 
        helpContentMenu = new JMenuItem(helpContentAction);
        aboutMenu  = new JMenuItem(aboutAction);
        
        fileMenu = new JMenu("File");
        fileMenu.setMnemonic(KeyEvent.VK_F);
        
        editMenu = new JMenu("Edit");
        editMenu.setMnemonic(KeyEvent.VK_E);        
        
        contactsMenu = new JMenu("Contacts");
        contactsMenu.setMnemonic(KeyEvent.VK_C);
        
        helpMenu = new JMenu("Help");
        helpMenu.setMnemonic(KeyEvent.VK_H);

        fileMenu.add( newFileMenu );
        fileMenu.add( openMenu );
        fileMenu.add( saveMenu );
        fileMenu.add( optionsMenu );
        fileMenu.add( exitMenu );

        editMenu.add( cutMenu );
        editMenu.add( copyMenu );
        editMenu.add( pasteMenu );
        
        contactsMenu.add( newContactMenu );
        contactsMenu.add( removeContactMenu );

        helpMenu.add( helpContentMenu );
        helpMenu.add( aboutMenu );
        
        menuBar.add(fileMenu);
        menuBar.add(editMenu);
        menuBar.add(contactsMenu);        
        menuBar.add(helpMenu);
    }

    public static void main(String[] args) {
        ContactLocale.load();
        ContactsFrame cf = new ContactsFrame();
        cf.setVisible(true);
    }
}
