/*
 * Created on 24 juin 2005
 */
package jgui.model;

import java.io.*;
import java.io.File;
import java.util.*;
import java.util.Vector;

import javax.swing.AbstractListModel;
import javax.swing.event.*;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import jgui.emailer.util.*;
import jgui.emailer.util.Contact;
import jgui.gui.PortraitRender;
import jgui.gui.beans.Portrait;

/**
 * @author Emmanuel DURIN
 *
 */
public class ContactsModel extends AbstractListModel implements TableModel{

    private Vector contacts = new Vector();
    private File f;
    private boolean isDirty;

    public int getSize() {
        return contacts.size();
    }

    public Object getElementAt(int index) {
        return contacts.get(index);
    }

    public void add( Contact c )
    {
        isDirty = true;        
        contacts.add(c);
        fireIntervalAdded(this,contacts.size()-1,contacts.size()-1);
        fireTableListeners(contacts.size()-1,contacts.size()-1,TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT);
    }
    
    public Object remove(int index) {
        isDirty = true;    
        if ( index >= 0 && index < contacts.size() ){
            isDirty = true;
            fireIntervalRemoved(this,index, index);
            fireTableListeners(index,index,TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE);
        }
        return contacts.remove(index);
    }
    public boolean remove(Contact c ) {
        int index = contacts.indexOf(c);
        if ( index != -1 ){
            isDirty = true;
            fireIntervalRemoved(this, index, index);
            fireTableListeners(index,index,TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE);            
        }
        return contacts.remove(c);
    }
    
    public void contactChanged( Contact c )
    {
        int index = contacts.indexOf(c);
        if ( index != -1 ){
            isDirty = true;
            fireContentsChanged(this, index, index);
            fireTableListeners(index,index,TableModelEvent.ALL_COLUMNS, TableModelEvent.UPDATE);
        }
    }
    public void save() throws IOException
    {
        Toolbox.saveVector(contacts, f);
        isDirty = false;
    }
    public void newFile()
    {
        f = null;
        isDirty = false;
        if (contacts != null){
            if ( contacts.size() > 0){ 
                fireIntervalRemoved(this,0,contacts.size()-1 );
                fireTableListeners(0,contacts.size()-1,TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE);
            }
	        contacts.clear();
        }
    }
    
    // Les exceptions doivent remonter jusqu' l'IHM
    // car on doit tre capable de les signaler  l'utilisateur
    public void open(File f) throws OptionalDataException, FileNotFoundException, ClassNotFoundException, IOException
    {
        newFile();
        isDirty = false;        
        this.f = f;
		contacts = Toolbox.loadVector( f );
		if ( contacts.size() > 0){
	        fireIntervalAdded(this,0,contacts.size()-1);
	        fireTableListeners( 0, contacts.size()-1,TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT);
		}
    }
    
    protected void fireTableListeners(int firstRow, int lastRow, int col, int eventType )
    {
	    for (Iterator iter = tablelisteners.iterator(); iter.hasNext();) {
	        TableModelListener listener = (TableModelListener)iter.next();
	        TableModelEvent event = new TableModelEvent(this,firstRow,lastRow,col, eventType);
	        listener.tableChanged(event);
	    }
    }
    
    public File getFile() {
        return f;
    }
    public void setFile(File selectedFile) {
        this.f = selectedFile;
    }    
    
    public boolean isDirty() {
        return isDirty;
    }
    public void setDirty(boolean isDirty) {
        this.isDirty = isDirty;
    }

    private static final Class [] tabClasses = { String.class, String.class, String.class, Portrait.class};
    private static final String [] columnNames = { "Firstname", "Lastname", "Email","Portrait"};
    private Vector tablelisteners = new Vector();
    
    public int getColumnCount() {
        return 4;
    }

    public int getRowCount() {
        return contacts.size();
    }
    public boolean isCellEditable(int row, int col) {
        return col < 3;
    }
    public Class getColumnClass(int col) {
        return tabClasses[col];
    }
    public Object getValueAt(int row, int col) {
        Contact c = (Contact)contacts.get(row);
        switch( col ){
        	case 0 :
        	    return c.getFirstname();
        	case 1 :
        	    return c.getLastname();
        	case 2 :
        	    return c.getEmailAddress();
        	case 3 :
        	    return new PortraitRender();        	    
        }
        return null;
    }

    public void setValueAt(Object o, int row, int col) {
        Contact c = (Contact)contacts.get(row);
        switch( col ){
        	case 0 :
        	    c.setFirstname( o.toString() );
        	    break;
        	case 1 :
        	    c.setLastname( o.toString() );
        	    break;
        	case 2 :
        	    c.setEmailAddress( o.toString() );
        	    break;
        }
        int index = contacts.indexOf(c);
        fireContentsChanged(c,index,index);
        // ????
        //fireTableListeners(index,index,TableModelEvent.ALL_COLUMNS,TableModelEvent.UPDATE);
    }
    
    
    public String getColumnName(int col) {
        return columnNames[col];
    }
    public void addTableModelListener(TableModelListener l) {
        tablelisteners.add(l);
        
    }
    public void removeTableModelListener(TableModelListener l) {
        tablelisteners.remove(l);        
    }
}