/*
 * Copyright (c) 2004 Michael Connor. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of FormLayoutBuilder or Michael Connor nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.mlc.swing.layout;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.dnd.*;
import java.awt.event.*;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import javax.swing.*;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.KeyStroke;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.event.*;
import javax.swing.table.DefaultTableCellRenderer;

/**
 * This is the pain panel that is used in LayoutFrame
 * serving as the user interface for the builder.
 * This is a pretty juicy file because there is a lot
 * going on here.  Better docs to come...
 *
 * @author Michael Connor mlconnor&#064;yahoo.com
 */
public class FormEditor extends JPanel {
    String[] verticalAlignmentList = {LayoutConstraintsManager.DEFAULT, LayoutConstraintsManager.FILL, LayoutConstraintsManager.CENTER, LayoutConstraintsManager.TOP, LayoutConstraintsManager.BOTTOM};
    String[] horizontalAlignmentList =
    {LayoutConstraintsManager.DEFAULT, LayoutConstraintsManager.FILL, LayoutConstraintsManager.CENTER, LayoutConstraintsManager.LEFT, LayoutConstraintsManager.RIGHT};
    ColSpanSpinnerModel colSpinnerModel = new ColSpanSpinnerModel();
    RowSpanSpinnerModel rowSpinnerModel = new RowSpanSpinnerModel();
    Action newComponentAction = new NewComponentAction();
    Action removeComponentAction = new RemoveComponentAction();
    Action insertRowBeforeAction = new InsertRowBeforeAction();
    Action insertRowAfterAction = new InsertRowAfterAction();
    Action deleteRowAction = new DeleteRowAction();
    Action insertColumnBeforeAction = new InsertColumnBeforeAction();
    Action insertColumnAfterAction = new InsertColumnAfterAction();
    Action deleteColumnAction = new DeleteColumnAction();
    JComboBox verticalAlignmentCombo = new JComboBox(verticalAlignmentList);
    JComboBox horizontalAlignmentCombo = new JComboBox(horizontalAlignmentList);
    JSpinner rowSpanSpinner = new JSpinner(rowSpinnerModel);
    JSpinner columnSpanSpinner = new JSpinner(colSpinnerModel);
    JLabel columnSpanLabel = new JLabel("Column Span");
    JLabel horizontalAlignmentLabel = new JLabel("Horizontal Alignment");
    JLabel rowSpanLabel = new JLabel("Row Span");
    JLabel verticalAlignmentLabel = new JLabel("Vertical Alignment");
    JPanel contentPanel = new JPanel();
    JPanel insetsPanel = new JPanel();
    SpinnerNumberModel rightInsetSpinnerModel = new SpinnerNumberModel(0, 0, Integer.MAX_VALUE, 1);
    SpinnerNumberModel topInsetSpinnerModel = new SpinnerNumberModel(0, 0, Integer.MAX_VALUE, 1);
    SpinnerNumberModel bottomInsetSpinnerModel = new SpinnerNumberModel(0, 0, Integer.MAX_VALUE, 1);
    SpinnerNumberModel leftInsetSpinnerModel = new SpinnerNumberModel(0, 0, Integer.MAX_VALUE, 1);
    JSpinner rightInsetSpinner = new JSpinner(rightInsetSpinnerModel);
    JSpinner bottomInsetSpinner = new JSpinner(bottomInsetSpinnerModel);
    JSpinner leftInsetSpinner = new JSpinner(leftInsetSpinnerModel);
    JSpinner topInsetSpinner = new JSpinner(topInsetSpinnerModel);
    GridTableModel tableModel = new GridTableModel();
    DnDTable table = new DnDTable();
    JScrollPane tableScrollPane = new JScrollPane(table);
    JLabel insetsLabel = new JLabel("Insets");
    JLabel componentsLabel = new JLabel("Components (Drag & Drop)");
    ComponentSelectionListModel componentSelectionListModel = new ComponentSelectionListModel();
    DndList componentList = new DndList(componentSelectionListModel);
    JScrollPane componentListScrollPane = new JScrollPane(componentList);
    ComponentListCellRenderer componentListCellRenderer = new ComponentListCellRenderer();
    Component constraintsSeparator = DefaultComponentFactory.getInstance().createSeparator("Component Constraints");
    Component positionsSeparator = DefaultComponentFactory.getInstance().createSeparator("Component Positions (Drag & Drop)");
    JPanel tablePanel = new JPanel();
    
    JTextField colSpecField = new JTextField();
    JTextField rowSpecField = new JTextField();
    
    java.util.Map <String, String> newComponents = new java.util.HashMap <String, String> ();
    LayoutConstraintsManager layoutConstraintsManager;
    JToolBar toolbar = new JToolBar();
    JButton newComponentButton = new JButton(newComponentAction);
    JButton removeComponentButton = new JButton(removeComponentAction);
    JButton columnDeleteButton = new JButton(deleteColumnAction);
    JButton columnInsertAfterButton = new JButton(insertColumnAfterAction);
    JButton columnInsertBeforeButton = new JButton(insertColumnBeforeAction);
    JButton rowDeleteButton = new JButton(deleteRowAction);
    JButton rowInsertBeforeButton = new JButton(insertRowBeforeAction);
    JButton rowInsertAfterButton = new JButton(insertRowAfterAction);
    
    Container container;
    ContainerLayout containerLayout;
    LayoutFrame layoutFrame;
    Component topComponent = null;
    boolean suspendConstraintControlUpdates = false;
    
    public FormEditor(LayoutFrame layoutFrame, ContainerLayout layout, Container container) {
        super();
        this.layoutFrame = layoutFrame;
        
        java.awt.LayoutManager layoutManager = container.getLayout();
        if ( ! (layoutManager instanceof ContainerLayout ) )
            throw new RuntimeException("Container layout must be of type ContainerLayout");
        
        containerLayout = (ContainerLayout) layoutManager;
        this.container = container;
        
        table.setBackground(java.awt.Color.white);
        table.setSelectionBackground(new Color(220, 220,255));
        table.setSelectionForeground(Color.black);

        table.setDefaultRenderer(Object.class, new ConstraintTableCellRenderer());
        table.setRowHeight(20);
        table.setModel(tableModel);
        table.setCellSelectionEnabled(true);
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        tableScrollPane.setPreferredSize(new Dimension(800,500));
        // let's put the cursor in the table so the create component
        // icon is enabled.  at least then the user knows what to do...
        if ( tableModel.getRowCount() > 1 &&
        tableModel.getColumnCount() > 1 )
            setSelectedCell(1,1);
        
        componentList.setCellRenderer(componentListCellRenderer);
        
        // let's setup all of the usability stuff...
        componentsLabel.setLabelFor(componentListScrollPane);
        //componentsLabel.setDisplayedMnemonic(KeyEvent.VK_S);
        verticalAlignmentLabel.setLabelFor(verticalAlignmentCombo);
        verticalAlignmentLabel.setDisplayedMnemonic(KeyEvent.VK_V);
        horizontalAlignmentLabel.setLabelFor(horizontalAlignmentCombo);
        horizontalAlignmentLabel.setDisplayedMnemonic(KeyEvent.VK_H);
        columnSpanLabel.setLabelFor(columnSpanSpinner);
        columnSpanLabel.setDisplayedMnemonic(KeyEvent.VK_C);
        rowSpanLabel.setLabelFor(rowSpanSpinner);
        rowSpanLabel.setDisplayedMnemonic(KeyEvent.VK_R);
        
        columnInsertAfterButton.setToolTipText("Insert a column after this column");
        columnInsertBeforeButton.setToolTipText("Insert a column before this column");
        columnDeleteButton.setToolTipText("Delete this column");
        rowInsertBeforeButton.setToolTipText("Insert a row before this row");
        rowInsertAfterButton.setToolTipText("Insert a row after this row");
        
        // let's setup the table toolbar
        toolbar.add(newComponentButton);
        toolbar.add(removeComponentButton);
        toolbar.addSeparator();
        toolbar.add(columnDeleteButton);
        toolbar.add(columnInsertBeforeButton);
        toolbar.add(columnInsertAfterButton);
        toolbar.addSeparator();
        toolbar.add(rowDeleteButton);
        toolbar.add(rowInsertBeforeButton);
        toolbar.add(rowInsertAfterButton);
        
        updateForm(); //updateFormControls();
        
        layoutConstraintsManager = LayoutConstraintsManager.getLayoutConstraintsManager(this.getClass().getResourceAsStream("editableLayoutConstraints.xml"));
        
        layoutConstraintsManager.setLayout("mainLayout", contentPanel);
        layoutConstraintsManager.setLayout("insetsLayout", insetsPanel);
        layoutConstraintsManager.setLayout("tableLayout", tablePanel);
        
        insetsPanel.add(rightInsetSpinner, "rightInsetSpinner");
        insetsPanel.add(leftInsetSpinner, "leftInsetSpinner");
        insetsPanel.add(topInsetSpinner, "topInsetSpinner");
        insetsPanel.add(bottomInsetSpinner, "bottomInsetSpinner");

        tablePanel.add(tableScrollPane, "tableScrollPane");
        tablePanel.add(componentListScrollPane, "componentListScrollPane");
        tablePanel.add(componentsLabel, "componentsLabel");
        
        contentPanel.add(rowSpanLabel, "rowSpanLabel");
        contentPanel.add(horizontalAlignmentCombo, "horizontalAlignmentCombo");
        contentPanel.add(horizontalAlignmentLabel, "horizontalAlignmentLabel");
        contentPanel.add(rowSpanSpinner, "rowSpanSpinner");
        contentPanel.add(verticalAlignmentCombo, "verticalAlignmentCombo");
        contentPanel.add(columnSpanLabel, "columnSpanLabel");
        contentPanel.add(verticalAlignmentLabel, "verticalAlignmentLabel");
        contentPanel.add(columnSpanSpinner, "columnSpanSpinner");
        contentPanel.add(insetsPanel, "insetsPanel");
        contentPanel.add(insetsLabel, "insetsLabel");
        contentPanel.add(constraintsSeparator, "constraintsSeparator");
        contentPanel.add(positionsSeparator, "positionsSeparator");
        contentPanel.add(tablePanel, "tablePanel");
        contentPanel.add(toolbar,"toolbar");        
        
        contentPanel.setBorder(Borders.DIALOG_BORDER);
        
        setLayout(new BorderLayout());
        add(contentPanel, BorderLayout.CENTER);
        
        setupListeners();
    }
    
    private void setupListeners() {
        
        verticalAlignmentCombo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Component component = getSelectedControl();
                if ( component != null ) {
                    CellConstraints cellConstraints = getComponentConstraints(component);
                    cellConstraints.vAlign = LayoutConstraintsManager.getAlignment((String) verticalAlignmentCombo.getSelectedItem());
                    updateLayout(component);
                }
            }
        });
        
        horizontalAlignmentCombo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Component component = getSelectedControl();
                if ( component != null ) {
                    CellConstraints cellConstraints = getComponentConstraints(component);
                    cellConstraints.hAlign = LayoutConstraintsManager.getAlignment((String) horizontalAlignmentCombo.getSelectedItem());
                    updateLayout(component);
                }
            }
        });
        
        topInsetSpinnerModel.addChangeListener( new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                if ( ! suspendConstraintControlUpdates ) {
                    Component component = getSelectedControl();
                    CellConstraints constraints = getComponentConstraints(component);
                    Insets insets = new Insets(topInsetSpinnerModel.getNumber().intValue(), constraints.insets.left, constraints.insets.bottom, constraints.insets.right);
                    constraints.insets = insets;
                    updateLayout(component);
                }
            }
        });
        
        leftInsetSpinnerModel.addChangeListener( new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                if ( ! suspendConstraintControlUpdates ) {
                    Component component = getSelectedControl();
                    CellConstraints constraints = getComponentConstraints(component);
                    Insets insets = new Insets(constraints.insets.top, leftInsetSpinnerModel.getNumber().intValue(), constraints.insets.bottom, constraints.insets.right);
                    constraints.insets = insets;
                    updateLayout(component);
                }
            }
        });
        
        rightInsetSpinnerModel.addChangeListener( new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                if ( ! suspendConstraintControlUpdates ) {
                    Component component = getSelectedControl();
                    CellConstraints constraints = getComponentConstraints(component);
                    Insets insets = new Insets(constraints.insets.top, constraints.insets.left, constraints.insets.bottom, rightInsetSpinnerModel.getNumber().intValue());
                    constraints.insets = insets;
                    updateLayout(component);
                }
            }
        });
        
        bottomInsetSpinnerModel.addChangeListener( new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                if ( ! suspendConstraintControlUpdates ) {
                    Component component = getSelectedControl();
                    CellConstraints constraints = getComponentConstraints(component);
                    Insets insets = new Insets(constraints.insets.top, constraints.insets.left, bottomInsetSpinnerModel.getNumber().intValue(), constraints.insets.right);
                    constraints.insets = insets;
                    updateLayout(component);
                }
            }
        });
        
        componentList.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {            
                if (e.getValueIsAdjusting() == false) {
                    Component component = (Component) componentList.getSelectedValue();
                    CellConstraints constraints = getComponentConstraints(component);
                    if ( component.isVisible() ) {
                        setSelectedCell(constraints.gridX, constraints.gridY);
                        topComponent = component;
                        table.repaint();
                        //tableModel.fireTableChanged( new javax.swing.event.TableModelEvent(tableModel, constraints.gridX, constraints.gridY));
                    }
                }
            }
        });    
    }
    
    public String getComponentDeclaration(String componentName) {
        return newComponents.get(componentName);
    }
    
    private String getComponentName(Component control) {
        return containerLayout.getComponentName(control);
    }
    
    private CellConstraints getComponentConstraints(Component component) {
        return containerLayout.getComponentConstraints(component);
    }
    
    private void specsChanged() {
        updateLayouts();
        
        // lets go down the tree
        Component[] children = container.getComponents();
        for ( int index = 0; index < children.length; index++ ) {
            Component component = children[index];
            if ( component instanceof Container )
                ((Container) component).doLayout();
        }
    }
    
    private void updateLayouts() {
        container.validate();
        container.doLayout();
        
        Container parent = container;
        
        while ( parent != null ) {
            parent.validate();
            parent = parent.getParent();
        }
    }
    
    private void setSelectedCell(int columnIndex, int rowIndex) {
        //System.out.println("setSelectedCell(" + columnIndex + ", " + rowIndex + ")");
        table.getSelectionModel().setSelectionInterval(rowIndex, rowIndex);
        table.getColumnModel().getSelectionModel().setSelectionInterval(columnIndex, columnIndex);
        
        // let's make sure the cell is in the visible range...
        JViewport viewport = (JViewport)table.getParent();
        Rectangle rect = table.getCellRect(rowIndex, columnIndex, true);
        Point pt = viewport.getViewPosition();
        rect.setLocation(rect.x-pt.x, rect.y-pt.y);
        viewport.scrollRectToVisible(rect);
        //System.out.println("end setSelectedCell(" + columnIndex + ", " + rowIndex + ")");
    }
    
    private class ComponentSelectionListModel extends javax.swing.AbstractListModel {
        private String selectedName = null;
        List<Component> sortedComponents = new ArrayList<Component>();
        
        public ComponentSelectionListModel() {
            super();
        }
        
        public Object getElementAt(int index) {
            Component component = sortedComponents.get(index);
            return component;
        }
        
        public int getSize() {
            sortedComponents = new ArrayList<Component>();
            
            if ( container != null ) {
                Component[] containerComponents = container.getComponents();
                for ( int index = 0; index < containerComponents.length; index++ ) {
                    Component insertComponent = containerComponents[index];
                    String insertComponentName = getComponentName(insertComponent);
                    
                    int insertIndex = 0;
                    while ( insertIndex < sortedComponents.size() && insertComponentName != null ) {
                        Component testComponent = sortedComponents.get(insertIndex);
                        String testName = getComponentName(testComponent);
                        if ( testName != null ) testName = testName.toUpperCase();
                        if ( insertComponentName.toUpperCase().compareTo(testName) <= 0 )
                            break;
                        else
                            insertIndex++;
                    }
                    sortedComponents.add(insertIndex, insertComponent);
                }
            }
            
            return container != null ? container.getComponentCount() : 0;
        }
        
        public void fireDelete() {
            super.fireContentsChanged(this, 0, Math.max(0, container.getComponents().length - 1));
        }
        
        public void fireInsert() {
            super.fireContentsChanged(this, 0, Math.max(0, container.getComponents().length - 1));
        }        
        
//        public Object getSelectedItem() {
//            return selectedName;
//        }
    }
    
    class ComponentListCellRenderer extends JLabel implements ListCellRenderer {
        Border noFocusBorder = new EmptyBorder(1, 1, 1, 1);
        
        public ComponentListCellRenderer() {
            super();
            setOpaque(true);
        }
        
        public Component getListCellRendererComponent(JList list, Object value, int index,
                                                      boolean isSelected, boolean cellHasFocus)
        {
            Component component = (Component) value;
            String name = getComponentName(component);

            setComponentOrientation(list.getComponentOrientation());
            if (isSelected) {
                setBackground(list.getSelectionBackground());
                setForeground(list.getSelectionForeground());
            } else {
                setBackground(list.getBackground());
                setForeground(list.getForeground());
            }

            setText(name != null ? name : "(Untitled)");
            
            setEnabled(list.isEnabled());
            Font font = list.getFont();
            setFont(font.deriveFont(component.isVisible() ? Font.PLAIN : Font.BOLD));
            setBorder((cellHasFocus) ? UIManager.getBorder("List.focusCellHighlightBorder") : noFocusBorder);
            
            return this;            
        }
    }    
    
    private void insertColumn(int column) {
        for ( int index = 0; index < container.getComponentCount(); index++ ) {
            Component component = container.getComponent(index);
            CellConstraints constraints = getComponentConstraints(component);
            if ( constraints.gridX > column )
                constraints.gridX++;
        }
        
        try {
            containerLayout.addColumnSpec(column, "pref");
            tableModel.fireTableStructureChanged();
            setSelectedCell(column + 1, 0);
            specsChanged();
        } catch (IllegalArgumentException iae) {
            JOptionPane.showMessageDialog(FormEditor.this, iae.getMessage(), "Invalid Layout", JOptionPane.ERROR_MESSAGE);
        }
    }
    
    private void insertRow(int rowIndex) {
        for ( int index = 0; index < container.getComponentCount(); index++ ) {
            Component component = container.getComponent(index);
            CellConstraints constraints = getComponentConstraints(component);
            if ( constraints.gridY > rowIndex )
                constraints.gridY++;
        }
        
        try {
            containerLayout.addRowSpec(rowIndex, "pref");
            tableModel.fireTableStructureChanged();
            setSelectedCell(0, rowIndex + 1);
            specsChanged();
        } catch (IllegalArgumentException iae) {
            System.out.println(iae);
            JOptionPane.showMessageDialog(FormEditor.this, iae.getMessage(), "Invalid Layout", JOptionPane.ERROR_MESSAGE);
        }
    }
    
    public void updateForm() {
        
        Component component = getSelectedControl();
        //System.out.println("updateForm(" + component + ")");
        
        CellConstraints constraints = component != null ? getComponentConstraints(component) : null;
        
        suspendConstraintControlUpdates = true;
        
        if ( component != null ) {
            rowSpinnerModel.setComponent(component);
            colSpinnerModel.setComponent(component);            
            verticalAlignmentCombo.setSelectedItem(LayoutConstraintsManager.getAlignment(constraints.vAlign));
            horizontalAlignmentCombo.setSelectedItem(LayoutConstraintsManager.getAlignment(constraints.hAlign));
            topInsetSpinnerModel.setValue(new Integer(constraints.insets.top));
            bottomInsetSpinnerModel.setValue(new Integer(constraints.insets.bottom));
            rightInsetSpinnerModel.setValue(new Integer(constraints.insets.right));
            leftInsetSpinnerModel.setValue(new Integer(constraints.insets.left));
        }
            
        verticalAlignmentCombo.setEnabled(constraints != null);
        horizontalAlignmentCombo.setEnabled(constraints != null);
        rightInsetSpinner.setEnabled(constraints != null);
        leftInsetSpinner.setEnabled(constraints != null);
        topInsetSpinner.setEnabled(constraints != null);
        bottomInsetSpinner.setEnabled(constraints != null);
        rowSpanSpinner.setEnabled(constraints != null);
        columnSpanSpinner.setEnabled(constraints != null);
            
        int col = table.getSelectedColumn();
        int row = table.getSelectedRow();
        
        newComponentAction.setEnabled( col > 0 && row > 0 );
        removeComponentAction.setEnabled( constraints != null );
        columnDeleteButton.setEnabled( row == 0 && col > 0 && containerLayout.getColumnCount() > 1);
        columnInsertAfterButton.setEnabled( col >- 1 );
        columnInsertBeforeButton.setEnabled( col > 0 );
        rowDeleteButton.setEnabled( col == 0 && row > 0 && containerLayout.getRowCount() > 1);
        rowInsertBeforeButton.setEnabled( row > 0 );
        rowInsertAfterButton.setEnabled( row > -1 );

        suspendConstraintControlUpdates = false;
        //System.out.println("exit updateForm(" + component + ")");
        
            //updateFormControls();  // maybe we could include this rather than call it...
    }
    
    public void updateLayout(Component component) {
        
        //System.out.println("updateLayout(" + component + ")");
        if ( suspendConstraintControlUpdates )
            return;
        
        CellConstraints constraints = getComponentConstraints(component);
        
        // we have to update the containerLayout which is the keeper of all
        // constraints.  if we didn't do this then we wouldn't notice any changes when
        // we went to print everything out.
        String name = getComponentName(component);
        // i don't like this direct access thing.  this should be changed...
        containerLayout.setCellConstraints(name, constraints);
//        updateForm();
        
        // be careful when modifying the next few lines of code.  this 
        // is tricky to get right.  this seems to work.
        container.invalidate();
        container.doLayout();
        
        if ( component instanceof Container )
        {
            Container cContainer = (Container) component;
            cContainer.invalidate();
            cContainer.doLayout();
        }
        
        //System.out.println("exit updateLayout(" + component + ")");
    }
    
    private Component getSelectedControl() {
        int col = table.getSelectedColumn();
        int row = table.getSelectedRow();
        Component component = ( row == 0 || col == 0 ) ? null : (Component) tableModel.getValueAt(row,col);
        //System.out.println("getSelectedControl(" + col + "," + row + ") = " + component);
        return component;
    }
    
    private class ColSpanSpinnerModel extends AbstractSpinnerModel {
        CellConstraints constraints;
        Component component;
        
        public ColSpanSpinnerModel() {
        }
        
        public void setComponent(Component component) {
            this.component = component;
            if ( component != null ) {
                constraints = getComponentConstraints(component);
                fireStateChanged();
            } else {
                constraints = null;
            }
        }
        
        public Object getNextValue() {
            if ( constraints == null )
                return null;
            Integer next = constraints.gridX + constraints.gridWidth - 1 < containerLayout.getColumnCount() ? new Integer(constraints.gridWidth + 1) : null;
            return next;
        }
        
        public Object getPreviousValue() {
            if ( constraints == null )
                return null;
            else {
                Integer previous = constraints.gridWidth > 1 ? new Integer(constraints.gridWidth - 1) : null;
                return previous;
            }
        }
        
        public Object getValue() {
            if ( constraints == null )
                return "";
            else
                return new Integer(constraints.gridWidth);
        }
        
        public void setValue(Object value) {
            if ( constraints == null || value == null )
                return;
            
            Number val = (Number) value;
            constraints.gridWidth = ((Number) value).intValue();
            super.fireStateChanged();
            updateLayout(component);
            tableModel.fireTableDataChanged();
        }
    }
    
    private class RowSpanSpinnerModel extends AbstractSpinnerModel {
        CellConstraints constraints;
        Component component;
        
        public RowSpanSpinnerModel() {
        }
        
        public void setComponent(Component component) {
            this.component = component;
            if ( component != null ) {
                constraints = getComponentConstraints(component);
                fireStateChanged();
            } else {
                constraints = null;
            }
        }
        
        public Object getNextValue() {
            if ( constraints == null )
                return null;
            else {
                Integer next = constraints.gridY + constraints.gridHeight - 1 < containerLayout.getRowCount() ? new Integer(constraints.gridHeight + 1) : null;
                return next;
            }
        }
        
        public Object getPreviousValue() {
            if ( constraints == null )
                return null;
            else {
                Integer previous = constraints.gridHeight > 1 ? new Integer(constraints.gridHeight - 1) : null;
                return previous;
            }
        }
        
        public Object getValue() {
            if ( constraints == null )
                return "";
            else
                return new Integer(constraints.gridHeight);
        }
        
        public void setValue(Object value) {
            if ( constraints == null || value == null )
                return;
            Number val = (Number) value;
            constraints.gridHeight = ((Number) value).intValue();
            super.fireStateChanged();
            updateLayout(component);
            tableModel.fireTableDataChanged();
        }
    }
    
    /**
     * Returns true if the named component was
     * created by hand in this session
     */
    public boolean isNewComponent(String name) {
        return newComponents.containsKey(name);
    }
    
    private class NewComponentDialog extends JDialog {
        JTextField nameField = new JTextField();
        JLabel nameLabel = new JLabel("Name");
        JLabel typeLabel = new JLabel("Type");
        JButton okButton = new JButton("OK");
        JButton cancelButton = new JButton("Cancel");
        JComboBox typeCombo = new JComboBox();
        PropertyTableModel propertyTableModel = new PropertyTableModel();
        JTable propertyTable = new JTable();
        JScrollPane propertyScrollPane = new JScrollPane(propertyTable);
        boolean wasSuccessful = false;
        Component component = null;
        Map <String, Object> controlProperties = new HashMap <String, Object> ();
        
        public NewComponentDialog(Frame owner)
        throws Exception {
            super(owner, "Add Component", true);
            
            okButton.setMnemonic(KeyEvent.VK_O);
            cancelButton.setMnemonic(KeyEvent.VK_C);
            nameLabel.setDisplayedMnemonic(KeyEvent.VK_N);
            typeLabel.setDisplayedMnemonic(KeyEvent.VK_T);
            nameLabel.setLabelFor(nameField);
            typeLabel.setLabelFor(typeCombo);
            
            FormLayout layout = new FormLayout("right:max(40dlu;pref), 3dlu, 130dlu", "");
            JPanel content = new JPanel();
            content.setBorder(Borders.DIALOG_BORDER);
            layoutConstraintsManager.setLayout("newComponentContent",  content);
            getContentPane().setLayout( new BorderLayout() );
            getContentPane().add(content, BorderLayout.CENTER);
            
            content.add(typeCombo, "typeCombo");
            content.add(nameField, "nameField");
            content.add(typeLabel, "typeLabel");
            content.add(nameLabel, "nameLabel");
            content.add(propertyScrollPane, "propertyScrollPane");
            content.add( com.jgoodies.forms.factories.ButtonBarFactory.buildRightAlignedBar(new JButton[]{ okButton, cancelButton}), "buttonPanel");
            
            propertyTable.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
            propertyTable.setModel(propertyTableModel);
            
            pack();
            
            typeCombo.addItem(new DefaultComponentBuilder(javax.swing.JButton.class, new String[] { "text" }));
            typeCombo.addItem(new DefaultComponentBuilder(javax.swing.JCheckBox.class, new String[] {"text"}));
            typeCombo.addItem(new DefaultComponentBuilder(javax.swing.JComboBox.class));
            typeCombo.addItem(new DefaultComponentBuilder(javax.swing.JLabel.class, new String[] {"text"}));
            typeCombo.addItem(new DefaultComponentBuilder(javax.swing.JPanel.class));
            typeCombo.addItem(new DefaultComponentBuilder(javax.swing.JPasswordField.class));
            typeCombo.addItem(new DefaultComponentBuilder(javax.swing.JRadioButton.class, new String[] {"text"}));
            typeCombo.addItem(new DefaultComponentBuilder(javax.swing.JScrollPane.class));
            typeCombo.addItem(new DefaultComponentBuilder(javax.swing.JSpinner.class));
            typeCombo.addItem(new JTableComponentBuilder());
            typeCombo.addItem(new DefaultComponentBuilder(javax.swing.JTextArea.class));
            typeCombo.addItem(new DefaultComponentBuilder(javax.swing.JTextField.class));
            typeCombo.addItem(new JToolBarComponentBuilder());
            typeCombo.addItem(new JTreeComponentBuilder());
            typeCombo.addItem(new SeparatorComponentBuilder());
            typeCombo.addItem(new ButtonBarComponentBuilder());
            
            typeCombo.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    controlProperties = new HashMap <String, Object>();
                    propertyTableModel.fireTableDataChanged();
                }
            });
            
            okButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    
                    try {
                        if ( nameField.getText().trim().length() == 0 )
                            throw new Exception("The name field is required");
                        
                        int currentCol = propertyTable.getSelectedColumn();
                        int currentRow = propertyTable.getSelectedRow();
                        
                        ComponentBuilder builder = (ComponentBuilder) typeCombo.getSelectedItem();
                        component = builder.getInstance(controlProperties);
                        
                        wasSuccessful = true;
                        dispose();
                        
                    } catch (Exception exception) {
                        exception.printStackTrace();
                        wasSuccessful = false;
                        JOptionPane.showMessageDialog(null, exception.getMessage(), "Error Creating Component", JOptionPane.ERROR_MESSAGE);
                    }
                }
            });
            
            cancelButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    wasSuccessful = false;
                    dispose();
                }
            });
        }
        
        public boolean wasSuccessful() {
            return wasSuccessful;
        }
        
        public String getComponentName() {
            return nameField.getText();
        }
        
        public String getComponentDeclaration() {
            ComponentBuilder builder = (ComponentBuilder) typeCombo.getSelectedItem();
            return builder.getDeclaration(getComponentName(), controlProperties);
        }
        
        public boolean isUsingLayoutComponent() {
            ComponentBuilder builder = (ComponentBuilder) typeCombo.getSelectedItem();
            return builder.isComponentALayoutContainer();
        }
        
        public Component getComponent() {
            return component;
        }
        
        private class PropertyTableModel extends javax.swing.table.AbstractTableModel {
            public int getColumnCount() {
                return 2;
            }
            
            public int getRowCount() {
                ComponentBuilder builder = (ComponentBuilder) typeCombo.getSelectedItem();
                return builder != null ? builder.getProperties().size() : 0;
            }
            
            public boolean isCellEditable(int row, int col) {
                return col == 1;
            }
            
            public String getColumnName(int col) {
                return col == 0 ? "Property" : "Value";
            }
            
            public void setValueAt(Object aValue, int row, int col) {
                ComponentBuilder builder = (ComponentBuilder) typeCombo.getSelectedItem();
                List <BeanProperty> properties = builder.getProperties();
                BeanProperty property = properties.get(row);
                controlProperties.put(property.getName(), aValue);
            }
            
            public Object getValueAt(int rowIndex, int columnIndex) {
                ComponentBuilder builder = (ComponentBuilder) typeCombo.getSelectedItem();
                List <BeanProperty> properties = builder.getProperties();
                BeanProperty property = properties.get(rowIndex);
                
                return columnIndex == 0 ? property.getName() : controlProperties.get(property.getName());
            }
            
            public Component getComponent()
            throws Exception {
                ComponentBuilder builder = (ComponentBuilder) typeCombo.getSelectedItem();
                Component instance = builder.getInstance(controlProperties);
                return instance;
            }
        }
    }
    
    private class ConstraintTableCellRenderer extends DefaultTableCellRenderer {
        public Component getTableCellRendererComponent(JTable table, Object value, 
            boolean isSelected, boolean hasFocus, int row, int column) {
                String stringValue = null;
                if ( value != null ) {
                    if ( value instanceof Component ) {
                        String name = getComponentName((Component) value);
                        stringValue = name == null ? "(Untitled)" : name;
                    } else {
                        // in this case it's a row or col header
                        stringValue = (String) value;
                    }
                }
                
                return super.getTableCellRendererComponent(table, stringValue, isSelected, hasFocus, row, column);
        }
    }
    
    private class GridTableModel extends javax.swing.table.AbstractTableModel {
        
        public int getColumnCount() {
            return containerLayout != null ? containerLayout.getColumnCount() + 1 : 1;
        }
        
        public int getRowCount() {
            return containerLayout != null ? containerLayout.getRowCount() + 1 : 1;
        }
        
        public boolean isCellEditable(int row, int col) {
            return ( row == 0 || col == 0 ) && ! ( row == 0 && col == 0 );
        }
        
        public String getColumnName(int col) {
            return col == 0 ? "*" : "" + col;
        }
        
        public void setValueAt(Object aValue, int row, int col) {
            String value = (String) aValue;
            if ( row == 0 ) // a column was changed
            {
                try {
                    containerLayout.setColumnSpec(col - 1, value);
                    specsChanged();
                } catch (IllegalArgumentException iae) {
                    JOptionPane.showMessageDialog(FormEditor.this, iae.getMessage(), "Invalid Layout", JOptionPane.ERROR_MESSAGE);
                }
            } else if ( col == 0 ) {
                try {
                    containerLayout.setRowSpec(row - 1, value);
                    specsChanged();
                } catch (Exception e) {
                    JOptionPane.showMessageDialog(null, e.getMessage(), "Invalid row specification", JOptionPane.ERROR_MESSAGE);
                }
            }
        }
        
        public Object getValueAt(int rowIndex, int columnIndex) {
            if ( rowIndex == 0 && columnIndex == 0 )
                return null;
            if ( rowIndex == 0 )
                return containerLayout.getColumnSpec(columnIndex - 1);
            if ( columnIndex == 0 )
                return containerLayout.getRowSpec(rowIndex - 1);
            
            Component component = null;
            
            for ( int index = 0; index < container.getComponentCount(); index++ ) {
                Component thisComponent = container.getComponent(index);
                // we don't want to show invisible components.  we
                // have decided to make components that are added without
                // constraints invisible until they are dropped onto the form.
                // this is our way of hiding them.
                if ( thisComponent.isVisible() ) {
                    CellConstraints constraints = getComponentConstraints(thisComponent);
                    if ( constraints == null )
                        throw new RuntimeException("Unable to find constraints for component " + thisComponent + " in layout " + containerLayout.getName());
                    if ( columnIndex >= constraints.gridX && columnIndex < constraints.gridX + constraints.gridWidth &&
                         rowIndex >= constraints.gridY && rowIndex < constraints.gridY + constraints.gridHeight ) {
                        component = thisComponent;
                        if (component == topComponent)
                            break;
                    }
                }
            }
            
            return component;
        }
    }
    
    private class RemoveComponentAction extends AbstractAction {
        public RemoveComponentAction() {
            super();
            putValue(Action.SHORT_DESCRIPTION, "Remove the component (Alt+D)");
            putValue(Action.LONG_DESCRIPTION, "Remove the component (Alt+D)");
            putValue(Action.SMALL_ICON, new ImageIcon(FormEditor.class.getResource("Remove24.gif")));
            putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_D, KeyEvent.CTRL_MASK));
        }
        
        public void actionPerformed(ActionEvent e) {
            Component selectedControl = getSelectedControl();
            String controlName = getComponentName(selectedControl);
            container.remove(selectedControl);
            tableModel.fireTableDataChanged();
            
            if ( selectedControl instanceof Container && layoutFrame.hasContainer(controlName) ) {
                layoutFrame.removeContainer(controlName);
            }
            container.doLayout();
            container.repaint();
            componentSelectionListModel.fireDelete();
            
            updateForm();
            table.requestFocus();
        }
    }
    
    private class NewComponentAction extends AbstractAction {
        public NewComponentAction() {
            super();
            putValue(Action.SHORT_DESCRIPTION, "Create a new component (Alt+N)");
            putValue(Action.LONG_DESCRIPTION, "Create a new component (Alt+N)");
            putValue(Action.SMALL_ICON, new ImageIcon(FormEditor.class.getResource("New24.gif")));
            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_N));
        }
        
        public void actionPerformed(ActionEvent e) {
            Frame frame = (Frame) SwingUtilities.getAncestorOfClass(Frame.class, FormEditor.this);
            
            NewComponentDialog newComponentDialog = null;
            int columnIndex = table.getSelectedColumn();
            int rowIndex = table.getSelectedRow();
            
            try {
                newComponentDialog = new NewComponentDialog(frame);
                newComponentDialog.setVisible(true);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            
            if ( newComponentDialog.wasSuccessful() ) {
                String controlName = newComponentDialog.getComponentName();
                Component newControl = newComponentDialog.getComponent();
                
                if ( containerLayout.getCellConstraints(controlName) != null ) {
                    JOptionPane.showMessageDialog(FormEditor.this, "A component named '" +
                    controlName + "' already exists", "Error", JOptionPane.ERROR_MESSAGE);
                } else {
                    // the best way to add this control is to setup the constraints
                    // in the map of name->constraints and then add it to the container.
                    // this layout manager will intercept it, find the constraints and then
                    // set it up properly in the table and assign the maps.
                    CellConstraints newConstraints = new CellConstraints(columnIndex, rowIndex);
                    containerLayout.getCellConstraints().put(controlName, newConstraints);
                    container.add(newControl, controlName);
                    
                    // we need to keep track of the new components added so we can present
                    // some code to the user to add back into their module.
                    newComponents.put(controlName, newComponentDialog.getComponentDeclaration());
                    
                    // if it's a panel, let's add it to the LayoutFrame so it can be manipulated too...
                    if ( newComponentDialog.isUsingLayoutComponent() ) {
                        Container newContainer = (Container) newControl;
                        layoutFrame.addContainer(controlName, newContainer);
                    }

                    componentSelectionListModel.fireInsert();
                    
                    //tableModel.fireTableDataChanged();
                    table.changeSelection(newConstraints.gridY,newConstraints.gridX,false,false);
//                    setSelectedCell(newConstraints.gridX, newConstraints.gridY);
                    updateLayouts();
                }
            }
            
            table.requestFocus();
        }
    }
    
    private class InsertRowBeforeAction extends AbstractAction {
        public InsertRowBeforeAction() {
            super();
            putValue(Action.SHORT_DESCRIPTION, "Inserts a row before the selected row");
            putValue(Action.LONG_DESCRIPTION, "Inserts a row before the selected row");
            putValue(Action.SMALL_ICON, new ImageIcon(FormEditor.class.getResource("RowInsertBefore24.gif")));
            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_I));
        }
        
        public void actionPerformed(ActionEvent e) {
            int row = table.getSelectedRow();
            insertRow(row-1);
            table.requestFocus();
        }
    }
    
    private class InsertRowAfterAction extends AbstractAction {
        public InsertRowAfterAction() {
            super();
            putValue(Action.SHORT_DESCRIPTION, "Inserts a row after the selected row");
            putValue(Action.LONG_DESCRIPTION, "Inserts a row after the selected row");
            putValue(Action.SMALL_ICON, new ImageIcon(FormEditor.class.getResource("RowInsertAfter24.gif")));
            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_O));
        }
        
        public void actionPerformed(ActionEvent e) {
            int row = table.getSelectedRow();
            insertRow(row);
            table.requestFocus();
        }
    }
    
    private class InsertColumnBeforeAction extends AbstractAction {
        public InsertColumnBeforeAction() {
            super();
            putValue(Action.SHORT_DESCRIPTION, "Inserts a column before the selected column");
            putValue(Action.LONG_DESCRIPTION, "Inserts a column before the selected column");
            putValue(Action.SMALL_ICON, new ImageIcon(FormEditor.class.getResource("ColumnInsertBefore24.gif")));
            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_K));
        }
        
        public void actionPerformed(ActionEvent e) {
            int column = table.getSelectedColumn();
            insertColumn(column-1);
            table.requestFocus();
        }
    }
    
    private class InsertColumnAfterAction extends AbstractAction {
        public InsertColumnAfterAction() {
            super();
            putValue(Action.SHORT_DESCRIPTION, "Inserts a column after the selected column");
            putValue(Action.LONG_DESCRIPTION, "Inserts a column after the selected column");
            putValue(Action.SMALL_ICON, new ImageIcon(FormEditor.class.getResource("ColumnInsertAfter24.gif")));
            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_L));
        }
        
        public void actionPerformed(ActionEvent e) {
            int column = table.getSelectedColumn();
            insertColumn(column);
            table.requestFocus();
        }
    }
    
    private class DeleteRowAction extends AbstractAction {
        public DeleteRowAction() {
            super();
            putValue(Action.SHORT_DESCRIPTION, "Deletes the selected row");
            putValue(Action.LONG_DESCRIPTION, "Deletes the selected row");
            putValue(Action.SMALL_ICON, new ImageIcon(FormEditor.class.getResource("RowDelete24.gif")));
            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_D));
        }
        
        public void actionPerformed(ActionEvent e) {
            int rowIndex = table.getSelectedRow();
            
            // move any components that are on the deleted row or
            // above it down one
            for ( int index = 0; index < container.getComponentCount(); index++ ) {
                Component component = container.getComponent(index);
                CellConstraints constraints = getComponentConstraints(component);
                
                if ( constraints.gridY >= rowIndex && constraints.gridY > 1) {
                    constraints.gridY--;
                } else {
                    // if the row deleted was within the span of the component and the component
                    // is bigger than one cell...
                    if ( constraints.gridY + constraints.gridHeight - 1 >= rowIndex && constraints.gridHeight > 1 )
                        constraints.gridHeight--;
                }
            }
            
            containerLayout.removeRowSpec(rowIndex - 1);
            
            tableModel.fireTableRowsDeleted(rowIndex, rowIndex);
            table.changeSelection (Math.min(rowIndex, containerLayout.getRowCount()), 0, false, false);
            specsChanged();
            table.requestFocus();
        }
    }
    
    private class DeleteColumnAction extends AbstractAction {
        public DeleteColumnAction() {
            super();
            putValue(Action.SHORT_DESCRIPTION, "Deletes the selected column");
            putValue(Action.LONG_DESCRIPTION, "Deletes the selected column");
            putValue(Action.SMALL_ICON, new ImageIcon(FormEditor.class.getResource("ColumnDelete24.gif")));
            putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_C));
        }
        
        public void actionPerformed(ActionEvent e) {
            int columnIndex = table.getSelectedColumn();
            
            for ( int index = 0; index < container.getComponentCount(); index++ ) {
                Component component = container.getComponent(index);
                CellConstraints constraints = getComponentConstraints(component);
                
                if ( constraints.gridX >= columnIndex && constraints.gridX > 1)
                    constraints.gridX--;
                else {
                    // if the col deleted was within the span of the component and the component
                    // is bigger than one cell...
                    if ( constraints.gridX + constraints.gridWidth - 1 >= columnIndex && constraints.gridWidth > 1 )
                        constraints.gridWidth--;
                }
            }
            
            containerLayout.removeColumnSpec(columnIndex - 1);
            tableModel.fireTableStructureChanged();
            table.changeSelection (0, Math.min(columnIndex, containerLayout.getColumnCount()), false, false);
            specsChanged();
            table.requestFocus();
        }
    }

    class DndList extends JList implements DragSourceListener, DragGestureListener {
        protected DragSource fDragSource = null;
        private Component draggingComponent = null;
        public DndList(ListModel listModel) {
            super(listModel);
            fDragSource = DragSource.getDefaultDragSource();
            fDragSource.createDefaultDragGestureRecognizer( this, DnDConstants.ACTION_MOVE, this );            
        }
        
        public void dragDropEnd(java.awt.dnd.DragSourceDropEvent dragSourceDropEvent) {}
        public void dragEnter(java.awt.dnd.DragSourceDragEvent dragSourceDragEvent) {}
        public void dragExit(java.awt.dnd.DragSourceEvent dragSourceEvent) {}
        public void dragOver(java.awt.dnd.DragSourceDragEvent dragSourceDragEvent) {}
        public void dropActionChanged(java.awt.dnd.DragSourceDragEvent dragSourceDragEvent) {}
        
        public void dragGestureRecognized( DragGestureEvent event ) {
            draggingComponent = (Component) super.getSelectedValue();
            if ( draggingComponent != null ) {
                event.startDrag(java.awt.dnd.DragSource.DefaultMoveDrop, new TransferableWrapper(draggingComponent), this);
            }
        }        
    }
    
    /**
     * I'm creating a table subclass to make it easier to handle dragging and dropping
     * http://www.hut.fi/~landerso/cccp/src/java/cccp/mappingtool/util/MTTable.java
     */
    class DnDTable extends JTable implements DragSourceListener, DragGestureListener, DropTargetListener, Autoscroll {
        protected DragSource fDragSource = null;
        protected DropTarget fDropTarget = null;
        
        final static int AUTOSCROLL_INSET_SIZE = 20;
        final static int SCROLL_AMOUNT = 10;
        
        public DnDTable() {
            super();
            fDragSource = DragSource.getDefaultDragSource();
            fDropTarget = new DropTarget(this, this);
            fDragSource.createDefaultDragGestureRecognizer( this, DnDConstants.ACTION_MOVE, this );
        }
        
        public void changeSelection (int rowIndex, int columnIndex, boolean toggle, boolean extend) {
            super.changeSelection(rowIndex, columnIndex, toggle, extend);
            updateForm();
        }
        
        /**
         * Implements autoscrolling.
         */
        public Insets getAutoscrollInsets() {
            Rectangle visible = getVisibleRect();
            Dimension size = getSize();
            int top = 0, left = 0, bottom = 0, right = 0;
            if (visible.y > 0)
                top = visible.y + AUTOSCROLL_INSET_SIZE;
            if (visible.x > 0)
                left = visible.x + AUTOSCROLL_INSET_SIZE;
            if (visible.y + visible.height < size.height)
                bottom = size.height - visible.y - visible.height + AUTOSCROLL_INSET_SIZE;
            if (visible.x + visible.width < size.width)
                right = size.width - visible.x - visible.width + AUTOSCROLL_INSET_SIZE;
            return new Insets(top, left, bottom, right);
        }
        
        /**
         * Implements autoscrolling.
         */
        public void autoscroll(Point cursorLocn) {
            Rectangle visible = getVisibleRect();
            int x =0, y = 0, width = 0, height = 0;
            // Scroll left.
            if (cursorLocn.x  < visible.x + AUTOSCROLL_INSET_SIZE) {
                x = -SCROLL_AMOUNT;
                width = SCROLL_AMOUNT;
            }
            // Scroll right.
            else if (cursorLocn.x  > visible.x + visible.width - AUTOSCROLL_INSET_SIZE) {
                x = visible.width + SCROLL_AMOUNT;
                width = SCROLL_AMOUNT;
            }
            // Scroll up.
            if (cursorLocn.y  < visible.y + AUTOSCROLL_INSET_SIZE) {
                y = -SCROLL_AMOUNT;
                height = SCROLL_AMOUNT;
            }
            // Scroll down.
            else if (cursorLocn.y  > visible.y + visible.height - AUTOSCROLL_INSET_SIZE) {
                y = visible.height + SCROLL_AMOUNT;
                height = SCROLL_AMOUNT;
            }
            ((JComponent) getParent()).scrollRectToVisible(new Rectangle(x, y, width, height));
        }
        
        public void dragEnter(DragSourceDragEvent event) {}
        public void dragOver(DragSourceDragEvent event) {}
        public void dropActionChanged(DragSourceDragEvent event) {}
        public void dragExit(DragSourceEvent event) {}
        public void dragDropEnd(DragSourceDropEvent event) {}
        
        public void dragGestureRecognized( DragGestureEvent event ) {
            Component component = getSelectedControl();
            
            if ( component != null ) {
                event.startDrag(java.awt.dnd.DragSource.DefaultMoveDrop, new TransferableWrapper(component), this);
            }
        }
        
        public void dragEnter(DropTargetDragEvent dropTargetDragEvent)
        {
            try {
                if ( dropTargetDragEvent.isDataFlavorSupported(new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType)) )
                    dropTargetDragEvent.acceptDrag(DnDConstants.ACTION_MOVE);
                else
                    dropTargetDragEvent.rejectDrag();
            } catch (ClassNotFoundException cnfe) {}     
        }
        
        public void dropActionChanged(DropTargetDragEvent event) {}
        public void dragExit(DropTargetEvent event) {}
        public void dragOver(java.awt.dnd.DropTargetDragEvent dropTargetDragEvent)
        {
            try {
                if ( dropTargetDragEvent.isDataFlavorSupported(new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType)) )
                    dropTargetDragEvent.acceptDrag(DnDConstants.ACTION_MOVE);
                else
                    dropTargetDragEvent.rejectDrag();
            } catch (ClassNotFoundException cnfe) {}
        }

        public void drop(java.awt.dnd.DropTargetDropEvent e) {
            
            try {
                java.awt.datatransfer.Transferable transferable = e.getTransferable();
                DataFlavor javaObject = new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType);
                if ( e.isDataFlavorSupported(javaObject) ) {
                    
                    Component component = (Component) transferable.getTransferData(javaObject);
                    CellConstraints cellConstraints = getComponentConstraints(component);
                    java.awt.Point location = e.getLocation();
                    int col = table.columnAtPoint(location);
                    int row = table.rowAtPoint(location);
                    if ( col > 0 && row > 0 ) {
                        cellConstraints.gridX = col;
                        cellConstraints.gridY = row;
                        cellConstraints.gridWidth = Math.min(cellConstraints.gridWidth, containerLayout.getColumnCount() - cellConstraints.gridX + 1);
                        cellConstraints.gridHeight = Math.min(cellConstraints.gridHeight, containerLayout.getRowCount() - cellConstraints.gridY + 1);
                        e.acceptDrop(DnDConstants.ACTION_MOVE);
                        e.getDropTargetContext().dropComplete(true);
                        if ( ! component.isVisible() )
                            component.setVisible(true);
                        topComponent = component; // make sure this sorts to the top...
                        updateLayout(component);
                        changeSelection (cellConstraints.gridY, cellConstraints.gridX, false, false);
                        repaint();
                        componentList.repaint();
                        //tableModel.fireTableDataChanged();
                    } else {
                        e.rejectDrop();
                        e.getDropTargetContext().dropComplete(true);
                    }
                } else {
                    e.rejectDrop();
                    e.dropComplete(false);
                }
            } catch(java.io.IOException ioe) {
                ioe.printStackTrace();
            } catch(java.awt.datatransfer.UnsupportedFlavorException ufe) {
                ufe.printStackTrace();
            } catch(ClassNotFoundException cnfe) {
                cnfe.printStackTrace();
            }
                
        }
    }
    
    class TransferableWrapper implements java.awt.datatransfer.Transferable
    {
        Object dragObject;
        
        public TransferableWrapper (Object dragObject) { this.dragObject = dragObject; }
        public synchronized Object getTransferData(DataFlavor flavor) { return dragObject; }
        public boolean isDataFlavorSupported (DataFlavor flavor) { return flavor.getMimeType().equals(DataFlavor.javaJVMLocalObjectMimeType); }
        public synchronized DataFlavor[] getTransferDataFlavors ()
        { 
            try {
                return new DataFlavor[] { new DataFlavor (DataFlavor.javaJVMLocalObjectMimeType) };
            } catch (ClassNotFoundException cnfe)
            {
                throw new RuntimeException ("Unable to load DataFlavor " + DataFlavor.javaJVMLocalObjectMimeType, cnfe);
            }
        }        
    }    
}