/*
 * Copyright (c) 2004 Michael Connor. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of FormLayoutBuilder or Michael Connor nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.mlc.swing.layout;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.FormLayout;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.filechooser.FileFilter;

/**
 * This is the frame that enables you to build a layout.
 * The principle component is the FormEditor panel.
 * 
 * @author Michael Connor mlconnor&#064;yahoo.com
 */
public class LayoutFrame extends JFrame
{
    LayoutConstraintsManager constraintsManager;
    JMenuBar menuBar = new JMenuBar();
    JMenu actionMenu = new JMenu("File");
    JMenuItem saveXML = new JMenuItem ("Save As");
    JMenuItem viewCode = new JMenuItem ("View Code");
    JMenuItem exit = new JMenuItem ("Exit");
    final JFileChooser fileChooser = new JFileChooser();
    Map <ContainerLayout, FormEditor> editors = new HashMap <ContainerLayout, FormEditor> ();
    JTabbedPane tabs = new JTabbedPane ();
    Map <ContainerLayout, Component> layoutToTab  = new HashMap <ContainerLayout, Component> ();
    List <ContainerLayout> newLayouts = new ArrayList <ContainerLayout> ();
    
    /** Creates a new instance of Class */
    public LayoutFrame (LayoutConstraintsManager constraintsManager)
    {
        super ("FormLayout Constraints Editor");

        if ( constraintsManager.getLayouts().size() == 0 )
            throw new RuntimeException("You must register at least one container by calling LayoutConstraintsManager.setLayout(String name, Container container) before instantiating a LayoutFrame");

        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        this.constraintsManager = constraintsManager;

        actionMenu.setMnemonic('F');
        saveXML.setMnemonic('A');
        saveXML.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, KeyEvent.CTRL_MASK));
        viewCode.setMnemonic('V');
        viewCode.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, KeyEvent.CTRL_MASK));
        exit.setMnemonic ('X');
        actionMenu.add(saveXML);
        actionMenu.add(viewCode);
        actionMenu.add(exit);
        menuBar.add(actionMenu);
        this.setJMenuBar(menuBar);        
        
        fileChooser.setFileFilter(new XmlFileFilter());

        this.addWindowListener(new WindowAdapter() 
        {
            public void windowClosing(WindowEvent e)
            {
                exitApplication();
            }            
        });
        
        exit.addActionListener( new ActionListener ()
        {
            public void actionPerformed (ActionEvent e)
            {
                exitApplication();
            }
        });
        
        List <ContainerLayout> layouts = constraintsManager.getLayouts();
        
        for ( int index = 0; index < layouts.size(); index++ )
        {
            ContainerLayout containerLayout = layouts.get(index);
            Container container = constraintsManager.getContainer(containerLayout);
            if ( container == null )
                throw new RuntimeException("A container with name " + containerLayout.getName() + " was found in the contstraints file but was not found in the container");
            addContainerLayout (containerLayout, container);
        }
        
        getContentPane().add (tabs, BorderLayout.CENTER);
        
        viewCode.addActionListener (new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                List <ContainerLayout> layouts = LayoutFrame.this.constraintsManager.getLayouts();
                StringBuffer declarationBuffer = new StringBuffer("// here are declarations for the controls you created\n");
                StringBuffer setLayoutBuffer = new StringBuffer("// here is where we load the layout constraints.  " + 
                    "change the xml filename!!!\norg.mlc.swing.layout.LayoutConstraintsManager layoutConstraintsManager " + 
                     "= \n    org.mlc.swing.layout.LayoutConstraintsManager.getLayoutConstraintsManager(\n        " + 
                     "this.getClass().getResourceAsStream(\"yourConstraintFile.xml\");\n");
                StringBuffer addBuffer = new StringBuffer("// here we add the controls to the container.  you may\n// " + 
                    "need to change the name of panel\n");
                
                for ( int index = 0; index < layouts.size(); index++ )
                {
                    ContainerLayout containerLayout = layouts.get(index);
                    FormEditor editor = editors.get (containerLayout);
                    Map <Component, String> componentsToNames = containerLayout.getComponentsToNames();
                    
                    for ( Iterator i = componentsToNames.keySet().iterator(); i.hasNext(); )
                    {
                        Component component = (Component) i.next();
                        String componentName = componentsToNames.get (component);
                        if ( editor.isNewComponent(componentName) )
                        {
                            declarationBuffer.append (editor.getComponentDeclaration(componentName));
                            addBuffer.append (containerLayout.getName() + ".add (" + componentName + ", \"" + componentName + "\");\n");
                        }
                    }
                    
                    if ( newLayouts.contains(containerLayout) )
                        setLayoutBuffer.append (containerLayout.getName() + ".setLayout(layoutConstraintsManager.createLayout (\"" + containerLayout.getName() +
                            "\", " + containerLayout.getName() + ");\n");
                }
                
                CodeDialog codeDialog = new CodeDialog (LayoutFrame.this, declarationBuffer.toString() + "\n" +  
                    setLayoutBuffer.toString() + "\n" + addBuffer.toString());
                codeDialog.setVisible(true);
            }
        });
        
        saveXML.addActionListener( new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                java.util.prefs.Preferences prefs = java.util.prefs.Preferences.userNodeForPackage(getClass());
		String pathString = prefs.get("lastpath",null);
		if ( pathString != null )
                {
                    File path = new File(pathString);
                    if ( path.exists() )
                    {
                        fileChooser.setCurrentDirectory(path);
                    }
                }

                int returnVal = fileChooser.showSaveDialog(LayoutFrame.this);
                
                if (returnVal == JFileChooser.APPROVE_OPTION)
                {
                    File file = fileChooser.getSelectedFile();

                    if ( file.exists() )
                    {
                        File path = file.getParentFile();
                        if ( path != null )
                        {
                            pathString = path.getAbsolutePath();
                            prefs.put("lastpath", pathString);
                        }


                        int result = JOptionPane.showConfirmDialog(LayoutFrame.this,
                            "The file you selected exists, ok to overwrite?", "File Exists", JOptionPane.YES_NO_OPTION);                       
                        if ( result != JOptionPane.YES_OPTION )
                            return;
                    }


                    
                    FileOutputStream outStream = null;
                    try
                    {
                        outStream = new FileOutputStream(file);
                        String xml = LayoutFrame.this.constraintsManager.getXML();
                        outStream.write(xml.getBytes());
                    } catch (Exception exception)
                    {
                        JOptionPane.showMessageDialog(LayoutFrame.this, "Error writing to file. " + exception.getMessage());
                        exception.printStackTrace();
                    } finally
                    {
                        try
                        {
                            if ( outStream != null )
                                outStream.close();
                        } catch (Exception ignore)
                        {
                        }
                    }
                }
            }
        });        
        
        pack();
    }
    
    public boolean hasContainer (String name)
    {
        return constraintsManager.getContainerLayout(name) != null;
    }
    
    private class XmlFileFilter extends FileFilter
    {
        
        public boolean accept(File f)
        {
            if ( f.isDirectory() )
                return true;
            
            String ext = null;
            String s = f.getName();
            int i = s.lastIndexOf('.');
            boolean isXml = false;
            
            if (i > 0 &&  i < s.length() - 1)
            {
                ext = s.substring(i+1).toLowerCase();
                isXml = ext.equals ("xml");
            }
            
            return isXml;

        }
        
        public String getDescription()
        {
            return "xml files";
        }
        
    }
    
    public void exitApplication ()
    {
        int result = JOptionPane.showConfirmDialog(LayoutFrame.this, "Are you sure you want to exit", "Exit Confirmation", JOptionPane.YES_NO_OPTION);
        if ( result == JOptionPane.YES_OPTION )
        {
            setVisible(false);
            System.exit(0);        
        }
    }
    
    public void removeContainer (String name)
    {
        ContainerLayout layout = constraintsManager.getContainerLayout(name);
        if ( layout == null )
            throw new RuntimeException ("Container " + name + " does not exist");
        constraintsManager.removeLayout(layout);
        FormEditor editor = editors.get (layout);
        tabs.remove (editor);
        newLayouts.remove (layout);
    }
    
    /**
     * This is for adding containers on the fly.  The idea is
     * that when someone creates a new panel in one of the existing
     * FormEditors, it can be added here and then they can lay it out.
     */
    public void addContainer (String name, Container container)
    throws IllegalArgumentException
    {
        // check to see if another panel with this name already exists
        ContainerLayout layout = constraintsManager.getContainerLayout(name);
        if ( layout != null )
            throw new IllegalArgumentException ("A container with name " + name + " already exists");
        
        layout = new ContainerLayout (name, "pref", "pref");
        constraintsManager.addLayout(layout);
        container.setLayout (layout);
        newLayouts.add (layout);
        
        addContainerLayout (layout, container);
    }
    
    private void addContainerLayout (ContainerLayout containerLayout, Container container)
    {
        FormEditor formEditor = new FormEditor (this, containerLayout, container);
        editors.put (containerLayout, formEditor);
        tabs.addTab (containerLayout.getName(), formEditor);
    }
    
    private class CodeDialog extends JDialog
    {
        public CodeDialog (Frame owner, String text)
        {
            super (owner, "Code", true);
            JPanel content = new JPanel();
            getContentPane().setLayout( new BorderLayout() );
            getContentPane().add(content, BorderLayout.CENTER);
            
            JTextArea textArea = new JTextArea ();
            textArea.setEditable(false);
            textArea.setText (text);
            textArea.setLineWrap(true);
            textArea.setWrapStyleWord(true);
            content.setLayout (new BorderLayout());
            
            JScrollPane areaScrollPane = new JScrollPane(textArea);
            areaScrollPane.setPreferredSize(new Dimension(600, 400));
            
            content.add (areaScrollPane, BorderLayout.CENTER);
            pack();
        }
    }    
    
    public static void main (String[] args)
    {
        JFrame frame = new JFrame();
        JPanel newComponentPanel = new JPanel();
        newComponentPanel.setBorder (Borders.DIALOG_BORDER);
        LayoutConstraintsManager constraintsManager = new LayoutConstraintsManager();
        constraintsManager.setLayout("panel", newComponentPanel);

        LayoutFrame layoutFrame = new LayoutFrame (constraintsManager);
        layoutFrame.setVisible(true);
        layoutFrame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        frame.getContentPane().add (newComponentPanel, BorderLayout.CENTER);
        frame.setSize(600,600);
//        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        frame.setVisible(true);        
    } 
}
